/*----------------------------------------------------------------
 *  ATeams
 *  Program to solve the Point-to-point connection problem
 *
 *  (C)Carlos A.S. Oliveira 1996-2001
 *----------------------------------------------------------------*/

/* ateams.c: general implementation of ATeams. */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "ateams.h"


#ifdef _WINDOWS_
#include <windows.h>
#include "ppc.h"
#endif /* _WINDOWS_ */

/* The global instance of the problem                       */
ppc_instance *gInst;
/* shared partial memory                                    */
partMem PartialMem;
/* name of the output file                                  */
char g_out_file[256] = "";
/* file from which to initialize the memory                 */
char gReadArq[256] = "";
/* Default output file                                      */
FILE *g_of = 0;
/* flag to initialize memory and dump to a file             */
int gInitMem = 0;
/* say if there is a limit to number of iterations          */
int gIterLimit = 0;
/* Flag to read the initialization from a file              */
int gReadMem = 0;
/* Flag that enables the verbose option (defaults to false) */
int gVerbose = 0;
/* last heuristic used                                      */
int g_heurist_chosen;
/* list of selected heuristics used                         */
int gHeuristList[MAX_NUM_HEURISTICS];
/* number of heuristics used in this run                    */
int gNumHeurist = 0;
/* Number of times the A-Teams is executed                  */
int gNumRepeat = 1;
/* The time of running for the A-Teams                      */
int gMaxTime = 3600;  /* set to 1 hour */
/* number of iteractions                                    */
int gNumIter = 0;
int gLastChangeIter = 0;
/* max number of iterations                                 */
int g_max_num_iter = 0;
/* The Optimum value for the instance (if it exists)        */
int gOptimum = 0;
/* value of best and worst solution                         */
int gBestSolValue, gWorstSolValue;
/* inicial time for A-Team                                  */
time_t gTimeIni;
/* pointers for list of results                             */
result_t *result, *last_res;
/* diz so o ateams deve parar                               */
int gStopAteams = 0;
/* true if we show when the worst solution is improved      */
int gPrintWorst = 0;
float gLB=0;
float bestLB=0;
/* used heuristics                                          */
t_heuristic heuristic[] = {
  /*{"C1",  0, Heuristic1},*/ /* this is used only in the initial phase */
  {"CB1", 0, Heuristic2},
  {"CB2", 0, Heuristic3},
  {"C3",  0, Heuristic4},
  {"I1",  0, Heuristic5},
  {"I2",  0, Heuristic6},
  {"I3",  0, Heuristic7},
  {"I4",  0, Heuristic8},
  {"JP1", 0, Heuristic9},
  {"JP2", 0, Heuristic10},
  {"RF",  0, Heuristic11},
  {"New", 0, Heuristic12}
};


/* insert a new solution, if it is acceptable */
void insert_new_solution(Solution *sol, Graph *G, int p, int pos)
{
  int custo;

  if (pos == -1)
    pos = get_delete_position(); /* position of the solution to be deleted */
  /* Include solution only if it's better then the wost and
     it's validated by the validate_solutionution function */
  custo = eval_solution (sol, G, p);
  if (custo < solucoes[worst_solution_pos].custo) {
    int i;
    /* Verify if sol is in solucoes */
    for (i = 0; i < gNumSolutions && custo < solucoes[i].custo; i++)
      ;
    if (custo == solucoes[i].custo && IsEqual(sol, solucoes[i].sol, p)){
      DeleteSolution(sol, p);
      return;
    }
    /* Insert the solution in order */
    protect_thread;
    insert_solution(sol, pos, custo);
    unprotect_thread;
  }
  else 
    DeleteSolution (sol, p);
}

/* Write the initializations to a file when the '-t' option is
   set in the command line. This creates a test file than a heuristic
   can be used in a isolated way */
static void write_initialization (FILE * f, int p)
{
  int i;

  fprintf (f, "INSTANCIA = %s\n", gInst->instName);
  fprintf (f, "NUMSOLUTIONS = %d\n", gNumSolutions);
  for (i = 0; i < gNumSolutions; i++) {
    int j;

    fputs ("#\n", f);
    for (j = 0; j < p; j++) {
      listvert *l;

      for (l = solucoes[i].sol[j].path; l; l = l->prox)
	fprintf (f, "%d ", l->v);
      fputs (" .\n", f);
    }
  }
  fputs ("#", f);
  fclose (f);
}

/* Read a initialization file (tipicaly when we are testing an
   heuristic). Used when the  `-r' option is specified in command line */
static void ReadInitialization (Graph * G, int p)
{
  FILE *f;
  int i, j;
  char str1[20], str2[20], str3[20];
  char erro[] = "Error reading inicialization file\n";
  
  f = fopen (gReadArq, "r");
  if (!f) {
    fatal_error("Nao foi possivel abrir o arquivo de inicializacao da"
		" memoria\n");
  }
  fscanf (f, "%s %s %s", str1, str2, str3);
  if (strcmp (str1, "INSTANCIA")
      || strcmp (str2, "=")
      || strcmp (gInst->instName, str3)) {
    fatal_error (erro);
  }
  fscanf (f, "%s %s %s", str1, str2, str3);
  if (strcmp (str1, "NUMSOLUTIONS")
      || strcmp (str2, "=")
      || gNumSolutions != atoi (str3)) {
    fatal_error (erro);
  }
  for (i = 0; i < gNumSolutions; i++) {
    solucoes[i].sol = NewVec (Solution, gInst->p);
    fscanf (f, "%s", str1);
    if (strcmp (str1, "#")) {
      fatal_error (erro);
    }
    for (j = 0; j < gInst->p; j++) {
      listvert *l;
      int lastVert;
      fscanf (f, "%s", str1);
      if (!strcmp (str1, ".")) {
	fatal_error (erro);
      }
      lastVert = (l = solucoes[i].sol[j].path =
		  new_list_vert())->v = atoi (str1);
      l->prox = NULL;
      fscanf (f, "%s", str1);
      while (strcmp (str1, ".")) {
	lastVert = (l = l->prox = new_list_vert())->v = atoi (str1);
	l->prox = NULL;
	fscanf (f, "%s", str1);
      }
      solucoes[i].sol[j].dest = lastVert;
    }
#ifndef NDEBUG      
    validate_solution(solucoes[i].sol);
#endif      
    solucoes[i].custo = eval_solution (solucoes[i].sol, G, p);
  }
  fclose(f);
}

void insert_initial_solution(Solution *sol, int cost, int i)
{
  int j, k;
  /* insert solution in order */
  for (j = 0; j < i; j++) {
    if (solucoes[j].custo >= cost) {
      /* shift old solutions */
      for (k = i; k > j; k--)
	solucoes[k] = solucoes[k - 1];

      /* insert the solution */
      solucoes[j].sol = sol;
      solucoes[j].custo = cost;
      return;
    }
  }
  /* no place was found */
  solucoes[i].sol = sol;
  solucoes[i].custo = cost;
}

void init_shared_memory(Graph *G, vert s[], vert d[], int p)
{
  int i, cost;
  Solution *sol;
  char mess[50];

  sprintf(mess, "Gerando %d solucoes\n", gNumSolutions);
  VERBOSE(mess);

  /* Fill the solutions memory whith random solutions */
  for (i = 0; i < gNumSolutions; i++) {
    sol = Heuristic1(G, s, d, p);
    cost = eval_solution (sol, G, p);
#ifndef NDEBUG    
    validate_solution(sol);
#endif    
    insert_initial_solution(sol, cost, i);
    
    sprintf(mess, "Nova solucao gerada [%d]\n", i+1);
    VERBOSE(mess);
  }
}

int get_num_heuristics()
{
  return sizeof(heuristic)/sizeof(t_heuristic);
}


/* Chose a ramdom heuristic to run */
int select_random_heuristic()
{
  if (gNumHeurist){			
    /* Run only the selected heuristics */
    int randomPos = random_int(gNumHeurist);
    return gHeuristList[randomPos];
  } else
    return random_int(get_num_heuristics());
}

void CheckInstanceBounds(FILE *f, int better)
{
  /* Print messages regarding to the the lower and upper bounds */
  if (gInst->hasBound) {
    char message[500];   /* message displayed */
    char cResult[10];   
    float perc =  ((gInst->Upper - better) /
		   ((float) gInst->Upper));
    if (perc >= 0) {
      /* The result was better than the upper bound */
      if (better < gInst->Lower){
	/* Very big trouble ... */
	sprintf (message, "Error: The result is below the lower" 
		 "bound!\n");
	fprintf(f, message);
      }
      strcpy(cResult, "below");
    } else {
      strcpy(cResult, "above");
    }
    /* The best solution is greater than the upper bound */
    sprintf (message, "The best solution is %s the" 
	     "upper bound in %5.2f %% \n", cResult, abs(perc));
    fprintf(f, message);
  }
}

/* Checks for various conditions that can finish ateams */
int can_stop(int best)
{
  /* The LowerBound was found (if it exists) */
  if (gInst->hasBound && gInst->Lower >= best)
    return 1;
  /* The Optimum for the problem was found
     (if it exists) */
  if (gOptimum && best==gOptimum)
    return 1;
  /* the max num of iterations was reached */
  if (gIterLimit && gNumIter > g_max_num_iter)
    return 1;
  /* verify the 'stop' flag of the user interface */ 
  protect_thread;
#ifdef _WINDOWS_
  return gStopAteams;
#endif  
  unprotect_thread;
  return 0;
}

/* read the instance file */
ppc_instance *read_instance(char *sFileName)
{
  ppc_instance *i;
  char c1[10], c2[10];
  FILE *f;
  int count;
  if (!(f = fopen (sFileName, "r+"))){
      printf("Error openning file %s.\n", sFileName);
      return 0;
  }
  i = New(ppc_instance);
  if (!i) {
    printf("Error: can not allocate memory for instance.\n");
    exit(1);
  }
  /* read the graph stored in the file */
  i->G = read_graph_from_file(f);
  if (!i->G){
    printf("error: can not read instance file\n");
    exit(1);
  }
  /* The number of source vertex */
  fscanf (f, "%s %s", c1, c2);
  if (strcmp (c1, "p") && strcmp (c1, "P")){
      printf("error reading number of source vertices\n");
      return 0;
  }
  i->p = atoi (c2);
  i->source = NewVec (int, i->p);
  fscanf (f, "%s", c1);
  if (strcmp (c1, "S")){
      printf("error reading source vertex\n");
      return 0;
  }

  /* Read the source vertices */
  for (count = 0; count < i->p; count++){
      fscanf (f, "%s", c1);
      i->source[count] = atoi (c1);
  }
  i->dest = NewVec (int, i->p);

  fscanf (f, "%s", c1);
  if (strcmp (c1, "D")){
      printf("error  reading number of destination vertices\n");
      return 0;
  }

  /* Read the destination vertices */
  for (count = 0; count < i->p; count++){
      fscanf (f, "%s", c1);
      i->dest[count] = atoi (c1);
  }
  i->hasBound = 0;

  /* See if this instance has upper and lower bounds */
  count = fscanf (f, "%s", c1);
  while (count > 0) {
    if (!strcmp(c1, "L")){
      /* Read the lower bound */
      i->hasBound = 1;
      fscanf (f, "%s", c1);
      i->Lower = atoi (c1);
      
      /* Read the upper bound */
      fscanf (f, "%s", c1);
      if (strcmp (c1, "U")){ 
	printf("error reading upper bound\n");
	return 0;
      }
      fscanf (f, "%s", c1);
      i->Upper = atoi (c1);
    } else if (!strcmp(c1, "H")){
      int value, tam = strlen (c1), j, digflag = 0;
      while (fscanf (f, "%s", c1) > 0){
	for (j = 0; !isdigit (c1[j]) && j < tam; j++)
	  digflag++;
	if (!digflag)
	  value = atoi (c1);
	if (value <= 0 && !digflag)
	  digflag++;
	if (digflag)
	  break;
	gHeuristList[gNumHeurist] = value;
	gNumHeurist++;
      }
      if (digflag)
	continue;
    } else if (!strcmp(c1, "I")) {
      fscanf(f, "%s", c1);
      g_max_num_iter = atoi(c1);
      if (g_max_num_iter < 1) {
	fatal_error("error in parameter I in the instance file\n");
      }
      gIterLimit     = 1;
    } else {
      printf("Parametro desconhecido em read_instance.\n");
      exit(0);
    }
    count = fscanf (f, "%s", c1);
  }
  /* Close the file */
  fclose (f);
  return i;
}


/* update the values of best and worst solutions.
   returns 1 if best solution changed,
   returns 2 if worst solution changed,
   returns 0 if nothing changed */
int update_best_solution()
{
  int iBest, iWorst;    /* index of best and worse solutions */

  iBest  = best_solution_pos;
  iWorst = worst_solution_pos;

  if ( solucoes[iBest].custo < gBestSolValue ) {
    gBestSolValue = solucoes[iBest].custo;
    last_res->next = New(result_t);
    if (last_res->next) {
      last_res = last_res->next;
      last_res->iter    = gNumIter;
      last_res->best    = gBestSolValue;
      last_res->heurist = g_heurist_chosen;
      last_res->next    = 0;
    }
    gLastChangeIter = gNumIter;
    return 1;
  }
  if ( solucoes[iWorst].custo < gWorstSolValue ) {
    gWorstSolValue = solucoes[iWorst].custo;
    return 2;
  }
  if ( gLB > bestLB ) {
    bestLB = gLB;
    return 1;
  }
  return 0; 
}

void end_ateam_execution(int p)
{
#ifdef _WINDOWS_
  WaitForSingleObject(gMutex, INFINITE);
  gATRunning = 0;
#else
  /*print_solution(g_of, solucoes[best_solution_pos].sol, p);*/
  VERBOSE("\nFinishing one exection...\n");
#endif
  DeleteSolutions(p);
  unprotect_thread;
}

void show_ateam_results(int changed)
{
#ifdef _WINDOWS_
  RedrawInfo();
#endif

  if (changed==1 || (changed==2 && gPrintWorst)){
#ifdef _WINDOWS_
    /* ask if we can update the Windows interface */
    if ( WaitForSingleObject(gMutex, 100)
	 == WAIT_OBJECT_0) {
      if (!gStopAteams)
	RedrawPPCWindow();
      ReleaseMutex(gMutex);
    }
#else
    fprintf(g_of, "%d\t%d\t%d\t%c%s\t%f\t%f\n",
	    gNumIter, gBestSolValue, gWorstSolValue,
	    (changed==1 ? '+' : '-'),
	    heuristic[g_heurist_chosen].h_name, gLB, bestLB);
#endif
  }
}

/* Initialize data structures, open files, and do some
   optional operations */
void init_ateam(Graph * G, vert s[], vert d[], int p)
{
  result = last_res = New(result_t);
  result->next = 0;

  /* initialize some data structures */
  init_solutions ();
  VERBOSE("Fiding all shortest paths\n");
  all_shortest_paths(gInst->G);
  /*{
    int i;
    for (i=1; i<=G_nVerts(gInst->G); i++)
      all_shortest_paths_from_vertex(gInst->G, i, A[i], Pi[i]);
  }*/
  VERBOSE("End of all shortest paths.\n");

  
  /* If we're reading the memory initialization from a file */
  if (gReadMem)
    ReadInitialization (G, p);
  else
    init_shared_memory(G, s, d, p);
  
  /* If we're only writing a initialization file, do it and exit */
  if (gInitMem) {
    write_initialization (g_of, p);
    exit (0);
  }
  /* Print the output file's header */
  fprintf(g_of, "n.iter\tBestSol.\tWorstSol.\tHeurist.\n");
}


void do_some_computation(Graph *G, vert s[], vert d[], int p)
{
  Solution *sol;
  int changed;

  g_heurist_chosen = select_random_heuristic();
  heuristic[g_heurist_chosen].n_used++;
  gNumIter++;

  sol = (*heuristic[g_heurist_chosen].func)(G, s, d, p);

#ifndef NDEBUG
  validate_solution(sol);
#endif

  insert_new_solution(sol, G, p, -1);

  protect_thread;
  changed = update_best_solution();
  unprotect_thread;

  show_ateam_results(changed);

  if (can_stop(gBestSolValue)){
    end_ateam_execution(p);
    gStopAteams = 1;
  }
  VERIFY_MEMORY;
}


int linearRandom (int x)
{
  return (int) pow (8 * long_rand(x * (x + 1) / 2.0) / 2.0, 0.5) + 1;
}
