/************************************************************************
 * gendifg.c                                                            *
 * generate ``difficult''  graphs for the heuristic I4                  *
 *                                                                      *
 * (C) 1999 Carlos Alberto                                              *
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include "graph.h"

#define LOW         1		/* peso minimo em uma aresta do grafo */
#define HIGH        10		/* peso maximo em uma aresta */

typedef enum {
    False = 0, True = 1
} BOOL;

struct edge {
  int v1;
  int v2;
};

void print_memory_error()
{
  printf ("error: insufficient memory\n");
  exit (0);
}

FILE *open_file (char filename[], char type[])
{
  FILE *fp;

  fp = fopen (filename, type);
  if (!fp){
    printf ("Erro na abertura do arquivo de saida\n");
    exit (0);
  }
  return fp;
}

/* ----------------------------------------------------------------------
   Generate numbers in uniform distribuition
   ---------------------------------------------------------------------- */
int Unif (long *seed, int low, int high)
{
  /* constants */
  long m = LONG_MAX;
  int a = 16807;
  long b = 127773;
  int c = 2836;
  int numb;
  double value_0_1;

  *seed = a * (*seed % b) - (*seed / b) * c;
  if (*seed < 0)
    *seed += m;
  value_0_1 = ((double) *seed) / m;
  numb = low + (int) (value_0_1 * (high - low + 1));
  return numb;
}

BOOL Pertence (int v1, int v2, struct edge ListEdge[], int tam)
{
  int i;

  for (i = 0; i < tam; i++)
    if (ListEdge[i].v1 == v1 && ListEdge[i].v2 == v2)
      return (True);

  return (False);
}

void Insere (int v1, int v2, struct edge ListEdge[], int indice)
{
  ListEdge[indice].v1 = v1;
  ListEdge[indice].v2 = v2;
}

int generate_random_graph(int argc, char **argv)
{
  FILE *fp;
  int i, k, peso, EdgeCount, *ListSourceDest, vertex, p;
  int v1, v2, temp, length, *ListVertices, n, m;
  long seed;
  char aux[10] = "", aux1[10] = "";
  char aresta[20] = "", Out[20] = "";
  struct edge *ListEdge;
  BOOL Achou;
  int *fontes, *destinos;

  if (argc != 5)
    {
      printf ("Usage: %s <#verts> <#arests> <#pairs> <#seed>\n", argv[0]);
      exit (0);
    }

  /* Recebe os paramentros */
  n = atoi (argv[1]);
  m = atoi (argv[2]);
  p = atoi (argv[3]);
  seed = atoi (argv[4]);
  sprintf (Out, "n%dm%dp%d.ins", n, m, p);

  fp = open_file(Out, "w+t");

  if (n < 2 * p) {
    printf ("error: num. of vertices must be at least \n"
	    "the number of pairs.\n");
    exit (0);
  }
  if (m > (n * n - n) / 2) {
    printf ("\nnum. of arests must be at most %d, n = %d, m = %d"
	    "p = %d", (n * n - n) / 2, n, m, p);
    exit (0);
  }

  if (!(fontes = (int *) malloc (sizeof (int) * p))
      || !(destinos = (int *) malloc (sizeof (int) * p)))
      print_memory_error();
  ListSourceDest = (int *) malloc (2 * p * sizeof (int));
  if (!ListSourceDest)
      print_memory_error();

  /* Gravando cabecalho do arquivo de saida */
  /* Numero de vertices */
  strcpy (aux, "n ");
  sprintf (aux1, "%d", n);
  strcat (aux1, "\n");
  strcat (aux, aux1);
  for (k = 0; k < strlen (aux); k++)
    fputc (aux[k], fp);

  /* Numero de arestas */
  strcpy (aux, "m ");
  sprintf (aux1, "%d", m);
  strcat (aux1, "");
  strcat (aux, aux1);
  for (k = 0; k < strlen (aux); k++)
    fputc (aux[k], fp);

  /* Vertices fontes */
  for (i = 0, length = 0; i < p; i++)
    {
      do
	{
	  Achou = True;
	  vertex = Unif (&seed, 1, n);
	  for (k = 0; k < i; k++)
	    if (ListSourceDest[k] == vertex)
	      {
		Achou = False;
		break;
	      }
	}
      while (!Achou);
      fontes[i] = vertex;
      ListSourceDest[i] = vertex;
    }

  /* Vertices Destinos */
  for (length = 0; i < 2 * p; i++)
    {
      do
	{
	  Achou = True;
	  vertex = Unif (&seed, 1, n);
	  for (k = 0; k < i; k++)
	    if (ListSourceDest[k] == vertex)
	      {
		Achou = False;
		break;
	      }
	}
      while (!Achou);

      destinos[i - p] = vertex;
      ListSourceDest[i] = vertex;
    }

  /* Determinando as arestas do grafo */
  ListEdge = (struct edge *) malloc (m * sizeof (struct edge));
  if (!ListEdge)
    print_memory_error();
  ListVertices = (int *) malloc (n * sizeof (int));
  if (!ListVertices)
    print_memory_error();
  strcpy (aux, "\na ");
  /* construindo um caminho hamiltoniano em G=(V,E) */
  ListVertices[0] = Unif (&seed, 1, n);
  for (i = 1, EdgeCount = 0; i < n; i++)
    {
      do
	{
	  Achou = True;
	  vertex = Unif (&seed, 1, n);
	  for (k = 0; k < i; k++)
	    if (vertex == ListVertices[k])
	      {
		Achou = False;
		break;
	      }
	}
      while (!Achou);
      ListVertices[i] = vertex;
      strcpy (aresta, aux);
      if (ListVertices[i - 1] > vertex)
	{
	  v1 = vertex;
	  v2 = ListVertices[i - 1];
	}
      else
	{
	  v1 = ListVertices[i - 1];
	  v2 = vertex;
	}
      sprintf (aux1, "%i", v1);
      strcat (aux1, " ");
      strcat (aresta, aux1);
      sprintf (aux1, "%i", v2);
      strcat (aux1, " ");
      strcat (aresta, aux1);
      peso = Unif (&seed, LOW, HIGH);
      Insere (v1, v2, ListEdge, EdgeCount);
      EdgeCount++;
      sprintf (aux1, "%i", peso);
      strcat (aresta, aux1);
      for (k = 0; k < strlen (aresta); k++)
	fputc (aresta[k], fp);
    }

  /* Arestas extras */
  strcpy (aux, "\na ");
  while (EdgeCount < m)
    {
      do
	{
	  v1 = Unif (&seed, 1, n);
	  v2 = Unif (&seed, 1, n);
	  if (v1 > v2)
	    {
	      temp = v1;
	      v1 = v2;
	      v2 = temp;
	    }
	  if (Pertence (v1, v2, ListEdge, EdgeCount))
	    v1 = v2;
	}
      while (v1 == v2);
      Insere (v1, v2, ListEdge, EdgeCount);
      EdgeCount++;
      strcpy (aresta, aux);
      sprintf (aux1, "%i", v1);
      strcat (aux1, " ");
      strcat (aresta, aux1);
      sprintf (aux1, "%i", v2);
      strcat (aux1, " ");
      strcat (aresta, aux1);
      peso = Unif (&seed, LOW, HIGH);
      sprintf (aux1, "%i", peso);
      strcat (aresta, aux1);
      for (k = 0; k < strlen (aresta); k++)
	fputc (aresta[k], fp);
    }

  fprintf (fp, "\nP %d\nS ", p);
  for (i = 0; i < p; i++)
    fprintf (fp, "%d ", fontes[i]);
  fprintf (fp, "\nD ");
  for (i = 0; i < p; i++)
    fprintf (fp, "%d ", destinos[i]);
  fclose (fp);
  return 0;
}


/* A ``difficult'' graph must have the following property:

	x + a1 + a2 > y > a1 + a2

   We create a graph with three partitions: a complete subgraph S
   with sources, D with destinations and the others. These two partitions
   are linked by chains of other vertices, from S to D.
   It will look like this:
           ___
     o ---'   `----  o
   o   o ----__----o   o
     o ----,___,---  o

  Arests in S have cost equal k, while the chains linking S to D have at
  least cost 3k. */


int with_clicks(int argc, char**argv) {
  int k, p, i, j;
  graph_node *n;
  Graph *G;
  char *usage = "usage: gendifg <k_value> <num of pairs> <out file>\n";
  
  if (argc != 3) {
    printf(usage);
    exit(1);
  }
  k = atoi(argv[1]);
  p = atoi(argv[2]);
  G = gr_create(4*p);
  /* first click */
  for (i=1; i<p; i++)
    for (j=i+1; j<=p; j++)
      gr_set_arest(G, i, j, k);
  /* arests to intermediate nodes */
  for (i=1; i<=p; i++)
    gr_set_arest(G, i, i+p, k);
  for (i=p+1; i<=2*p; i++)
    gr_set_arest(G, i, i+p, k);
  for (i=2*p+1; i<=3*p; i++)
    gr_set_arest(G, i, i+p, k);
  /* second click */
  for (i=3*p+1; i<4*p; i++)
    for (j=i+1; j<=4*p; j++)
      gr_set_arest(G, i, j, k);
  
  /* write graph to file */
  printf("n %d\nm %d\n", 4*p, p*(p-1)+3*p);
  for (i=1; i<=4*p; i++) 
    for (n=G->node[i].prox; n; n=n->prox) 
      if (i < n->v) 
	printf("a %d %d %d\n", i, n->v, n->custo);
  printf("P %d\nS ", p);
  for (i=1; i<=p; i++)
    printf("%d ", i);
  printf("\nD ");
  for (i=3*p+1; i<=4*p; i++)
    printf("%d ", i);
  printf("\n");
  return 0;
}


/* - Parameters:
   n = number of vertices
   m = number of arests
   p = number of pairs source/destination
   c = number of layers between the source and destination subgraphs
   r = the range of weights for arests;

   - Algorithm:
   Create 3 subgraphs: one with source vertices, other with destination
   vertices and the third with intermediate vertices.
*/

#define param_error(s,d) \
do { \
  printf("%s must be at least %d\n", s, (d)); \
  exit(1); \
} while (0) 

#define  calc_weight \
  max_w - min_w == 0 ? min_w : \
    min_w + rand() % (max_w - min_w + 1)

int main(int argc, char**argv) {
  int         a, b, c, i, j, l, m, n, p, r, w;
  int         seed, min_m, min_n, min_w, max_w;
  graph_node *no;
  Graph      *G;
  FILE       *f;
  char       *usage = "usage:\n gengraph <#pairs> <#layers> <seed>"
    " [<#vertices> <#arests>\n    <#min_weight> <#max_weight>]\n"
    " or gengraph -clicks <#weight> <#pairs>\n"
    " or gengraph -random <#verts> <#arests> <#pairs> <#seed>\n";

  if (argc < 4 || argc > 8) {
    printf(usage);
    exit(1);
  }
  if (!strcmp(argv[1], "-clicks"))
    return with_clicks(argc-1, &argv[1]);
  if (!strcmp(argv[1], "-random"))
    return generate_random_graph(argc-1, &argv[1]);
  /*------ #pairs ------------*/
  p = atoi(argv[1]);
  if (p < 2)
    param_error("pairs", 2);
  /*------ #layers -----------*/
  l = atoi(argv[2]);
  if (l < 2)
    param_error("layers", 2);
  /*----- seed ---------------*/
  seed = atoi(argv[3]);
  srand(seed);
  /*------ #vertices ---------*/
  min_n = (2+l)*p;
  if (argc >= 5){
    n = atoi(argv[4]);
    if (n < min_n)
      param_error("vertices", min_n);
  } else
    n = min_n;
  /*-------#arests -----------*/
  min_m = 2*(p-1) + p + (n-2*p);
  if (argc >= 6) {
    m = atoi(argv[5]);
    if (m < min_m || m > (n*(n-1))/2)
      param_error("arests", min_m);
  } else
    m = min_m;
  /*-------#min weight--------*/
  if (argc >= 7) {
    min_w = atoi(argv[6]);
    if (min_w < 1)
      param_error("min weight", 1);
  } else 
    min_w = max_w = 10 + rand()%10;
  /*-------#max weight--------*/
  if (argc >= 8) {
    max_w = atoi(argv[7]);
    if (max_w < min_w)
      param_error("max weight", min_w);
  }
  else
    max_w = min_w;
  
  /* atualiza l */
  l = (m - 2*p)/p;
  G = gr_create(n);
  /* create source and destination subgraphs */
  for (i=1; i<p; i++) {
    w = calc_weight;
    gr_set_arest(G, i, i+1, w);
  }
  for (i=n-p+1; i<n; i++) {
    w = calc_weight;
    gr_set_arest(G, i, i+1, w);
  }

  /* ensure that source, intermediate and destination subgraphs are
     interconnected */
  for (i=1; i<=p; i++){
    for (j=i; j<=n-2*p; j=j+p){
      w = calc_weight;
      gr_set_arest(G, j, j+p, w);
    }
  }
  for (i=0; i<p; i++){
    w = calc_weight;
    gr_set_arest(G, n-i, n-p-i, w);
  }
  
  /* insert adicional arests */
  if ((m - gr_narests(G))/3 <= ((p-2)*(p-1))/2)
    c = (m - gr_narests(G))/3;
  else
    c = ((p-2)*(p-1))/2;
  /* insert in the source subgraph */
  for (i=0; i<c; i++){
    do { 
      a = 1 + rand() % p;
      do {
	b = 1 + rand() % p;
      } while (a == b);
    } while (gr_get_cost(G, a, b) != 0);
    w = calc_weight;
    gr_set_arest(G, a, b, w);
  }
  /* insert in the middle subgraph */
  r = m - gr_narests(G) - c;
  for (i=0; i<r; i++){
    do { 
      a = p + 1 + rand() % (n-2*p);
      do {
	b = p + 1 + rand() % (n-2*p);
      } while (a == b);
    } while (gr_get_cost(G, a, b) != 0);
    w = calc_weight;
    gr_set_arest(G, a, b, w);
  }
  /* insert in the destination subgraph */
  for (i=0; i<c; i++){
    do { 
      a = (n-p) + 1 + rand() % p;
      do {
	b = (n-p) + 1 + rand() % p;
      } while (a == b);
    } while (gr_get_cost(G, a, b) != 0);
    w = calc_weight;
    gr_set_arest(G, a, b, w);
  }
  /* write graph to file */
  printf("n %d\nm %d\n", n, m);
  for (i=1; i<=n; i++)
    for (no=G->node[i].prox; no; no=no->prox)
      if (i < no->v)
	printf( "a %d %d %d\n", i, no->v, no->custo);
  printf("P %d\nS ", p);
  for (i=1; i<=p; i++)
    printf( "%d ", i);
  printf("\nD ");
  for (i=n-p+1; i<=n; i++)
    printf( "%d ", i);
  printf("\n");
  fclose(f);
  return 0;
}

