/*******************************************************************
 * Arquivo: GRAFOLA.H
 * Objetivo: Define um TAD para grafos usando listas de adjacencias
 *
 * C.A - 11/JAN/95
 ********************************************************************/

#ifndef _GRAFO_H
#define _GRAFO_H

#include <math.h>
#include <stdlib.h>
#include "common.h"

#ifndef abs
#define abs(x)  (x>0 ? x : -x)
#endif

/* uncomment if you want to use adj. list */
#define ADJLIST
/* uncomment if you want to use forward star */
/*#define FORWARDSTAR*/

typedef int vert;

typedef struct _graph_node {
  int custo;
  vert v;
  struct _graph_node *prox;
} graph_node;

/* estrutura de dados Grafo usando foward-star */
typedef struct {
  int *v;       /* vetor de indices para os vertices */
  int *a;       /* vetor de arestas */
  int *w;       /* vetor de pesos das arestas */
  int nVerts;   /* numero de vertices no grafo */
  int nArests;  /* numero de arestas no grafo */
  int nArSize;  /* tamanho do vetor de arestas alocado */
} fs_graph;

#if !defined(FORWARDSTAR) && !defined(ADJLIST)
#error You must define FORWARDSTAR or ADJLIST!
#endif

#ifdef FORWARDSTAR


typedef fs_graph Graph;

#define gr_create	grfs_create
#define gr_delete	grfs_delete
#define gr_set_arest	grfs_set_arest
#define gr_set_arest_dir grfs_set_arest_dir
#define gr_remove_edge	grfs_remove_edge
#define gr_remove_edge_dir grfs_remove_edge_dir
#define gr_get_cost	grfs_get_cost
#define gr_get_total_cost grfs_get_total_cost
#define gr_vdegree	grfs_vdegree
#define gr_copy	        grfs_copy
#define gr_get_adj_list	grfs_get_adj_list
#define gr_nverts(G)    (G)->nVerts
#define gr_narests(G)	(G)->nArests
#define gr_print	grfs_print
#define gr_remove_all_edges grfs_remove_all_edges
#define gr_set_graph_matrix grfs_set_graph_matrix

#endif

#ifdef ADJLIST

typedef struct { 
  graph_node *node;
  int **A;
} Graph;

#define gr_create	gral_create
#define gr_delete	gral_delete
#define gr_set_arest	gral_set_arest
#define gr_set_arest_dir gral_set_arest_dir
#define gr_remove_edge	gral_remove_edge
#define gr_remove_edge_dir gral_remove_edge_dir
#define gr_get_cost	gral_get_cost
#define gr_get_total_cost gral_get_total_cost
#define gr_vdegree(G,v)	((G)->node[(v)].custo)
#define gr_copy		gral_copy
#define gr_get_adj_list	gral_get_adj_list
#define gr_nverts(G)	((G)->node[0].v)
#define gr_narests(G)	((G)->node[0].custo)
#define gr_print	gral_print
#define gr_remove_all_edges gral_remove_all_edges
#define gr_set_graph_matrix gral_set_graph_matrix

#endif /* ADJLIST     */


/* create a new graph */
Graph* grfs_create (int);
/* Deleta a mem'oria associada ao grafo */
void grfs_delete(Graph*);
void grfs_print(Graph *G);
/* Adiciona uma aresta a um grafo nao direcionado */
int grfs_set_arest(Graph*, vert, vert, int);
int grfs_set_arest_dir(Graph * G, vert a, vert b, int cost);
/* Funcao que retorna o custo de uma aresta */
int grfs_get_cost(Graph *,vert , vert);
int grfs_get_total_cost(Graph * G);
/* Funcao que le um grafo a partir de um arquivo externo */
Graph *grfs_Read(FILE *f);
/*  Retorna o grau de um vertice */
int grfs_vdegree(Graph *G, vert v);
/* Retorna a copia de um grafo */
Graph *grfs_copy(Graph *);
/* create a list of adjacencies to the vertex v. */
graph_node *grfs_get_adj_list(Graph *G, vert v);
void grfs_remove_all_edges(Graph *G);

/* Inicializa o gra'fo */
Graph* gral_create (int);
/* Deleta a mem'oria associada ao grafo */
void gral_delete(Graph*);
void gral_print(Graph *G);
int gral_remove_edge(Graph *G, vert a, vert b);
int gral_remove_edge_dir(Graph *G, vert a, vert b);
/* Adiciona uma aresta a um grafo nao direcionado */
int gral_set_arest(Graph*, vert, vert, int);
int gral_set_arest_dir(Graph * G, vert a, vert b, int cost);
/* Funcao que retorna o custo de uma aresta */
int gral_get_cost(Graph*,vert , vert);
int gral_get_total_cost(Graph * G);
/* Retorna a copia de um grafo */
Graph *gral_copy(Graph *G);
/* create a list of adjacencies to the vertex v. */
graph_node *gral_get_adj_list(Graph *G, vert v);
void gral_remove_all_edges(Graph *G);
int gral_set_graph_matrix(Graph *G);

/* delete a list of nodes */
void delete_list_of_nodes(graph_node *h);
void gr_return_adj_list(graph_node *n);
int gr_forest_cost(Graph *G);

void delete_graph_node(graph_node *l);
graph_node *new_graph_node();
Graph *read_graph_from_file(FILE * f);

/* Imprime uma mensagem de erro e sai do programa */
long int lrandomico(long int i);
int linearRandom(int x);
int LinearRand(int n);

#endif  /* _GRAFO_H */




























