/*#include <config.h>*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "ateams.h"

#ifdef _WINDOWS_  /* this file has windows code only */

#include <windows.h>
#include "ppc.h"

/*  -----------------------------
 *  Variaveis globais
 *  ----------------------------- */

/* instancia do programa */
extern HINSTANCE ghInst;

/* nome da classe da janela */
static char *strWClass = "PPCInfo";

/* fonte usada pelo programa */
extern HFONT gPPCFont;


/*  -----------------------------
 *  Prototipos de funcoes
 *  ----------------------------- */

LRESULT CALLBACK 
PPCInfoProc(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp);

static int
InitPPCInfo(HWND hWnd);

static void
Paint(HWND hWnd);

void
CreateLabel(HWND pai, char *title, int style, 
		int id, int x1, int y1, int x2, int y2);

static void
UpdateInfoItens(HWND hWnd, HDC dc);

static void
UpdateInfo(HWND hWnd);




/*  -----------------------------
 *  Implementacao de funcoes
 *  ----------------------------- */

/* 
 *  FUNCTION: CreatePPCW
 *  PURPOSE:  procedimento da janela que contem o grafo
 *  PARAMETERS:
 *  RETURN:
 *	padroes para um procedimento de janela
 */
LRESULT CALLBACK 
PPCInfoProc(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp)
{
	switch (mess){
	case WM_CREATE:
		return InitPPCInfo(hWnd);

	case WM_PAINT:
		if (GetUpdateRect(hWnd, NULL, TRUE)){
			Paint(hWnd);
		}
		break;

	case WM_UPDATEINFO:
		UpdateInfo(hWnd);
		break;

	default:
		return DefWindowProc(hWnd, mess, wp, lp);
	}
	return 0;        
}

/* 
 *  FUNCTION: RegisterPPCW
 *  PURPOSE: registra a janela do grafo. 
 *  PARAMETERS:
 */
void
RegisterPPCInfo()
{
	WNDCLASS wc;
	wc.style = CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = PPCInfoProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = ghInst;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) (COLOR_BTNFACE + 1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = strWClass;
	/* Registra a janela */
	RegisterClass(&wc);
}

/* 
 *  FUNCTION: CreatePPCInfo
 *  PURPOSE:  cria a janela de informacoes
 *  PARAMETERS:
 *	parent - janela pai 
 *	x1, y1, x2, y2 - posicao e tamanho iniciais
 */
HWND 
CreatePPCInfo(HWND parent, int x1, int y1, int x2, int y2)
{
	return CreateWindowEx(WS_EX_CLIENTEDGE, strWClass, "", 
		WS_CHILD | WS_BORDER | WS_VISIBLE,x1, y1, x2,y2, 
		parent, NULL, ghInst, NULL);
}

static void
CreateLabelEx(HWND pai, char *title, int style,  
		int id, int x1, int y1, int x2, int y2)
{
	HWND label;
	label = CreateWindowEx(WS_EX_CLIENTEDGE, "STATIC", title,
		WS_CHILD | WS_VISIBLE | style,
		x1, y1, x2, y2, pai, (HMENU)id, ghInst, NULL);
	SendMessage(label, WM_SETFONT, (WPARAM)gPPCFont, MAKELPARAM(TRUE, 0));		
}

static void
CreateLabelBold(HWND pai, char *title, int style,  
		int id, int x1, int y1, int x2, int y2)
{
	HWND label;
	LOGFONT plf;
	HFONT font;

	memset(&plf, 0, sizeof(LOGFONT));
	plf.lfHeight = 15;
	strcpy(plf.lfFaceName, "Arial"); 
	plf.lfWeight = FW_BOLD; 
	plf.lfEscapement = 0;
	font = CreateFontIndirect(&plf);
	label = CreateWindow("STATIC", title,
		WS_CHILD | WS_VISIBLE | style,
		x1, y1, x2, y2, pai, (HMENU)id, ghInst, NULL);
	SendMessage(label, WM_SETFONT, (WPARAM)font, MAKELPARAM(TRUE, 0));		
}

void
CreateLabel(HWND pai, char *title, int style, 
		int id, int x1, int y1, int x2, int y2)
{
	HWND label;
	label = CreateWindow("STATIC", title,
		WS_CHILD | WS_VISIBLE | style,
		x1, y1, x2, y2, pai, (HMENU)id, ghInst, NULL);
	SendMessage(label, WM_SETFONT, (WPARAM)gPPCFont, MAKELPARAM(TRUE, 0));		
}

int
InitPPCInfo(HWND hWnd)
{
	int i, x = 30, n;

	CreateLabelBold(hWnd, "Agents", SS_CENTER, 0, 2, 10, 88, 14);
	for (i=0, n = get_num_heuristics(); i<n; i++, x+=23){
		/* cria um label para a heuristica */
		CreateLabel(hWnd, heuristic[i].h_name, SS_RIGHT,
			    0, 5, x, 30, 14);
	}

	/* cria labels para  o melhor e pior valor */
	CreateLabelBold(hWnd, "Solutions:", 0, 0, 5, x += 12, 88, 16);	
	CreateLabel(hWnd, "Best:", 0, 0, 5, x+=23, 88, 16);
	CreateLabelEx(hWnd, "", SS_RIGHT | WS_BORDER, 1, 10, x += 19, 70, 18);
	CreateLabel(hWnd, "Worst:", 0, 0, 5, x += 22, 88, 16);
	CreateLabelEx(hWnd, "", SS_RIGHT | WS_BORDER, 2, 10, x += 19, 70, 18);
	return 0;
}

void
Paint(HWND hWnd)
{
	HDC dc;
	PAINTSTRUCT p;

	BeginPaint(hWnd, &p);
	dc = p.hdc;
	UpdateInfoItens(hWnd, dc);	
	EndPaint(hWnd, &p);	
}


void
UpdateInfo(HWND hWnd)
{
	HDC dc = GetDC(hWnd);
	UpdateInfoItens(hWnd, dc);
	ReleaseDC(hWnd, dc);
}

void
UpdateInfoItens(HWND hWnd, HDC dc)
{
	int i, n, x;
	RECT r;
	char value[30];
	HBRUSH red = CreateSolidBrush(RGB(255,0,0));       /* vermelho */
	HWND wnd;
	int last, best, worst;
	int update = 0;


	WaitForSingleObject(gMutex, INFINITE);
	if (gATRunning){
		last = g_heurist_chosen;
		best = gBestSolValue;
		worst = gWorstSolValue;
		update = 1;
	}
	ReleaseMutex(gMutex);

	/* desenha os quadrados */
	for (i=0, x=25, n=get_num_heuristics(); i<n; i++, x+=23){
		SetRect(&r, 45, x+3, 61, x+20);
		DrawEdge(dc, &r, EDGE_SUNKEN, BF_RECT | BF_MIDDLE);
	}
	if (update){
		int pos = last * 23 + 30;		
		sprintf(value, "%d", best);
		wnd = GetDlgItem(hWnd, 1);
		SetWindowText(wnd, value);

		sprintf(value, "%d", worst);
		wnd = GetDlgItem(hWnd, 2);
		SetWindowText(wnd, value);

		SetRect(&r, 47, pos, 59, pos + 13);
		FillRect(dc, &r, red);			
	}
}

#if 0
ResetInfoValues()
{
	int last, best, worst, update = 0;
	int i, x = 10;

	WaitForSingleObject(gMutex, INFINITE);
	if (!gATRunning){
		g_heurist_chosen = 0;
		gBestSolValue = 0;
		gWorstSolValue = 0;
		update = 1;
	}
	ReleaseMutex(gMutex);
	if (update){
		int pos = last * 20 + 15;
		sprintf(value, "%d", best);
		wnd = GetDlgItem(hWnd, 1);
		SetWindowText(wnd, value);

		sprintf(value, "%d", worst);
		wnd = GetDlgItem(hWnd, 2);
		SetWindowText(wnd, value);

		SetRect(&r, 47, pos, 57, pos + 10);
		FillRect(dc, &r, red);		
	}
}
#endif 

#endif  /*_WINDOWS_*/
