/************************************************************************
 * LISTVERT.C
 * Useful functions to manipulate lists of verts
 *
 * C.A. & Allan - MAI/96
 ************************************************************************/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ateams.h"

/* Memory buffer to allocate lisverts */
typedef struct {
  listvert *lista;
  struct blocks *block;	/* list of alocated blocks */
} memBuffer;			


/* a block of listverts */
struct blocks {
  listvert *start;
  struct blocks *prox;
};

/* Define a static memory buffer to listvert */
memBuffer buffer;

static listvert **mark;

/* Size of the memory used (number of solutions in main memory */
static long int MEMTAM;


/* Initialize the linked list */
void init_list_vert_buffer()
{
  int i, temp;
  MEMTAM = 
    (temp = (gr_nverts(gInst->G) * gNumSolutions * gInst->p )/10) <= 1000 
    ? 1000 
    : temp ;
  
  buffer.lista = NewVec(listvert, MEMTAM);
  if (!buffer.lista) {
    fatal_error("Falta memoria para buffer");
  }

  /* links all the listverts together */
  for (i=0; i<MEMTAM-1; i++)
      buffer.lista[i].prox = &buffer.lista[i+1];

  buffer.lista[MEMTAM-1].prox = NULL;
  /* aloca memoria para a lista de blocos */
  buffer.block = New(struct blocks);
  if (!buffer.block) {
    fatal_error("Falta memoria para lista de blocos");
  }
  buffer.block->start = buffer.lista;
  buffer.block->prox = NULL;
}

/* Deallocates all memory used to listverts
 */
void delete_list_vert_buffer()
{
  struct blocks *b = buffer.block, *t;
  
  while (b) {
    Delete(b->start);
    t = b->prox;
    Delete(b);
    b = t;
  }
}

void
InitListvertModule()
{
  int numVert = gr_nverts(gInst->G);
  mark = NewVec(listvert*, numVert+1);
  if (!mark) {
    fatal_error("Problemas de alocacao de memoria em Delete Loop\n");
  }
}

void
DeleteListvertModule()
{
  Delete(mark);
}

/* Gets a new listvert from the memory pool.
   The listverts are allocated in a block, placed in buffer.lista.
   If the block is empty, a new one is alocated, and placed in a 
   linked list. 
   The linked list is used to delete all the listverts allocated at
   the end of the program. */
listvert *new_list_vert()
{
  listvert *mem; static int flag=1;

  if (flag){
    /* Make the initialization of the blocks. It's called
       only at the first use of the function */
    init_list_vert_buffer(); 
    flag=0; 
  }

  /* the list of listverts is exausted? */
  if (buffer.lista == NULL) { 
    /* Allocates more memory */
    int i;
    struct blocks *b;
     
    VERBOSE("alocando mais memoria para listverts!!!\n");
    buffer.lista = NewVec(listvert, MEMTAM);
    if (!buffer.lista) {
      fatal_error("Memoria insuficiente para um novo listvert\n");
    }

      /* link all the listverts of the block */
    for (i=0; i<MEMTAM-1; i++)
      buffer.lista[i].prox = &buffer.lista[i+1];

    buffer.lista[MEMTAM-1].prox = NULL;

    /* encadeia o novo bloco encontrado na lista de blocos alocados */
    b = New(struct blocks);
    if (!b) {
      fatal_error("Memoria insuficiente para um novo struct buffer\n");
    }
    b->start = buffer.lista;
    b->prox = buffer.block;
    buffer.block = b;
  }
  mem = buffer.lista;
  buffer.lista = buffer.lista->prox;
  mem->prox = NULL;
  
  return mem;
}



/* Returns the listvert to the memory pool */
void delete_list_vert(listvert *l){
  l->prox = buffer.lista;
  buffer.lista = l;
}


/* Print the list of vertex in a path */
void print_list(listvert *s){
  listvert *l = s;
  while (l) {
    printf("%d,",l->v);
    l = l->prox;
  }
  printf("\n");
}

/* Find loops in a path and delete them */
int delete_loop(listvert *path, Graph *G)
{
  int n = gr_nverts(G), ret_val = 0, v;
  listvert *l, *aux, *temp;

  memset(mark, 0, sizeof(mark[0])*(n+1));
  for (l = path; l;) {
    /* if l->v was marked, we must remove the existent loop */
    if (mark[l->v]) { 
      v = l->v;
      aux = mark[v]->prox;
      l = l->prox;
      /* Delete until find the position after the 
	 repeated vertex*/
      while (aux != l) { 
	/* remove mark for other vertices */
	if (aux->v != v) {
	  mark[aux->v] = 0;
	}
	temp = aux->prox;
	delete_list_vert(aux);
	aux = temp;
      }
      mark[v]->prox = l;
      ret_val = 1;
    } else {  
      mark[l->v] = l;
      l = l->prox;
    }
  }
  return ret_val;
}
