/************************************************************************
 * ATEAMS.C
 * File that implement a execution system to the Assincronous-Teams
 *
 * 
 * (C) 1996, 1997 Carlos Alberto
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <string.h>
#include "ateams.h"

static char *gsHelp[] =
{
  "Commands:",
  "  -f\t\tOutput file name",
  "  -n\t\tRun A-Teams n times",
  "  -time\t\tMax time of execution",
  "  -v\t\tVerbose mode",
  "  -opt\t\tOptimum value for the instance",
  "  -t\t\tOnly initialize the memory and write to a file",
  "  -r\t\tReads solution memory from file",
  "  -h <list of heuristics>\tExecute only heuristics in list",
  "  -i <num>\tmax number of iterations",
  "  -help\t\tShow this help message",
  "  -msize <num>\tsize of A-Team memory",
  ""
};

/* program presentation string */
char *gsPurpose[] =
{
  "Ateams - solve the Point-to-point conection Problem using",
  "  Assincronous Teams",
  "(C) 1996, 1997 Carlos Alberto & Allan Reffson",
  ""
};

/* program use string */
static char *gsUse[] =
{
  "use:",
  "\tppc <InstanceFile> [-f <OutFile> -time <timeSeconds>",
  "\t-opt <optimumValue> -v]",
  ""
};

/* processes the args in commmand line */
void ProcessCommandLineArgs (int n, char *arg[])
{
  int i;

  if (!strcmp (arg[1], "-help")){
    /* prints only a help message and exit */
    mlprint (gsHelp);
    exit (0);
  }

  for (i = 2; i < n; i++){
    int flagt = 0, flagf = 0;

    if ((flagf = !strcmp (arg[i], "-f")) ||
	(flagt = !strcmp (arg[i], "-t"))){
      if (i + 1 > n) {
	/* Exits whit a error */
	printf ("Voce precisa especificar o arquivo apos %s\n",
		flagf ? "-f" : "-t");
	exit (1);
      }
      /* records the output file name */
      strcpy (g_out_file, arg[i + 1]);
      i++;
      if (flagt){
	if (gReadMem) {
	  fatal_error("Flag '-t' nao pode ser usado com '-r'\n");
	}
	gInitMem = 1;
      }
    } else if (!strcmp (arg[i], "-n")) {
      int j, digflag = 0, tam = strlen (arg[i]);

      i++;
      for (j = 0; !isdigit (arg[i][j]) && j < tam; j++)
	digflag++;
      if (digflag) {
	fatal_error("O Parametro n precisa de um valor inteiro\n");
      }
      gNumRepeat = atoi (arg[i]);

      /* We impose a pratical limit to n */
      if (gNumRepeat < 1 || gNumRepeat > 100) {
	fatal_error("Valor errado para -n\n");
	gNumRepeat = 0;
      }
    } else if (!strcmp (arg[i], "-r")) {
      if (gInitMem) {
	fatal_error("Flag '-r' nao pode ser usado com '-t'\n");
      }
      if (i + 1 > n) {
	fatal_error ("Voce precisa especificar um arquivo para leitura!\n");
      }
      strcpy (gReadArq, arg[i + 1]);
      gReadMem = 1;
      ++i;
      continue;
    } else if (!strcmp (arg[i], "-h")) {
      if (gInitMem) {
	fatal_error ("Flag desconhecido\n");
      }
      for (i++; i < n; i++) {
	char num[5];
	int j, tam = strlen (arg[i]), digflag = 0, numValue;
	strcpy (num, arg[i]);

	for (j = 0; !isdigit (num[j]) && j < tam; j++)
	  digflag++;
	/* Se um dos digitos for nao numerico, volte a analisa-lo */
	if (digflag) {
	  --i;
	  break;
	}
	if (((numValue = atoi (num)) >= get_num_heuristics() ||
	     numValue < 0)) {
	  printf ("O numero da heuristica precisa estar entre 0 e %d\n",
		  get_num_heuristics());
	  exit (0);
	}
	gHeuristList[gNumHeurist] = atoi (num);
	gNumHeurist++;
      }
    } else if (!strcmp (arg[i], "-time")) {
      int tam = strlen (arg[i + 1]), j, digflag = 0, time;

      i++;
      for (j = 0; !isdigit (arg[i][j]) && j < tam; j++)
	digflag++;
      if (!digflag)
	time = atoi (arg[i]);
      if (time <= 0 && !digflag)
	digflag++;
      if (digflag) {
	fatal_error ("\nO Parametro para 'time' esta errado");
      }
      gMaxTime = time;

    } else if (!strcmp (arg[i], "-i")) {
      int tam = strlen (arg[i + 1]), j, digflag = 0, niter;

      i++;
      for (j = 0; !isdigit (arg[i][j]) && j < tam; j++)
	digflag++;
      if (!digflag)
	niter = atoi (arg[i]);
      if (niter <= 0 && !digflag)
	digflag++;
      if (digflag) {
	fatal_error ("\nO Parametro para '-i' esta errado");
      }
      g_max_num_iter = niter;
      gIterLimit     = 1;
    } else if (!strcmp (arg[i], "-opt")) {
      int tam = strlen (arg[i + 1]), j, digflag = 0, opt;

      i++;
      for (j = 0; !isdigit (arg[i][j]) && j < tam; j++)
	digflag++;
      if (!digflag)
	opt = atoi (arg[i]);
      if (time <= 0 && !digflag)
	digflag++;
      if (digflag) {
	fatal_error ("\nO Parametro para 'opt' esta errado");
      }
      gOptimum = opt;
    } else if (!strcmp (arg[i], "-v")) {
      gVerbose = 1;
    } else if (!strcmp(arg[i], "-msize")) {
      i++;
      gNumSolutions = atoi(arg[i]);
      gNumSolutions = gNumSolutions%2 == 0 ? gNumSolutions :
                      gNumSolutions+1;
      if (gNumSolutions <= 0) {
        printf("msize must be at least 1\n");
	gNumSolutions = 0;
      }
    } else {
      printf ("Argumento invalido na linha de comando: %s\n", arg[i]);
      exit (0);
    }
  }
}

#include "rf.h"

int main (int n, char **arg)
{
  int j = 0;
  gTimeIni = time (NULL);
  if (n < 2) {
    mlprint (gsUse);
    exit (0);
  }
  if (!strcmp(arg[1], "-mst")) {
    return mst_bound(n-1, &arg[1]);
  }
  if (!strcmp(arg[1], "-rf")) {
    return rf_bound(n-1, &arg[1]);
  }
  /* processes the args in commmand line */
  ProcessCommandLineArgs(n, arg);
  /* Read the instance file */
  gInst = read_instance(arg[1]);
  if (!gInst)
    printf("error: I cannot read the instance file %s\n", arg[1]);


  strcpy(gInst->instName, arg[1]);
  /* initialize the random generator */
  srand ((int) time(NULL));
  /* open output file */
  if (*g_out_file) {
    g_of = fopen (g_out_file, "w+");
    if (!g_of) {
      printf ("error openning file %s\n", g_out_file);
      exit (1);
    }
  } else  /* We'll use the standard output */ 
    g_of = stdout;

  if (gNumSolutions == 0) {
    gNumSolutions = gr_nverts(gInst->G);
    gNumSolutions = gNumSolutions%2 == 0 ?  gNumSolutions : 
                    gNumSolutions + 1;
    gNumSolutions = (gNumSolutions)*2;
  }

  /* main loop of application */
  for (j = 0; j < gNumRepeat; j++) {
    /* execute the A-Teams */
    run_ateam(gInst->G, gInst->source, gInst->dest, gInst->p);
  }

  gr_delete (gInst->G);
  printf("End of processing. Total time: %ds\n",
	 (int)difftime (time (NULL), gTimeIni));
  return 0;  
}

void run_ateam(Graph *G, vert s[], vert d[], int p)
{
  /* Initialize data structures, open files, etc. */
  init_ateam(G, s, d, p);

  protect_thread;
  gBestSolValue  = solucoes[best_solution_pos].custo;
  gWorstSolValue = solucoes[worst_solution_pos].custo;
#ifdef _WINDOWS_
  gATRunning = 1;
  EnablePPCButton(IDM_GRAPH_STOP, TRUE);
#endif
  unprotect_thread;

  /* main loop -  run agents assyncronously */
  while (!gStopAteams)
    do_some_computation(G, s, d, p);
}


