/************************************************************************
 * ppcmaster.c - Master program for the parallel Asynchronous Team
 * for the point-to-point connection problem
 *
 * (C) 1999 - Carlos Alberto de S. Oliveira
 ************************************************************************/

#include <stdio.h> 
#include <string.h> 
#include <stdlib.h>
#include <time.h>
#include <pvm3.h>
#include "sinc.h"

#define nmax 10
#define Q HOME RELATIVE_PATH "sincq"

int read_config_file(char *CFile);

int nproc, n;
char PFile[80], ParamFile[80];
int Delta[nmax][nmax];

 
int main(int argc, char *argv[]){
  int mytid, neig[nmax], i, k;
  int contt,cont;
  div_t qtpp;                   /* number of nodes per processor */

  mytid = pvm_mytid();
  /*pvm_catchout(stdout);*/

  srand(time(NULL)*mytid);
  if(argc!=2) {
    
    printf("usage: ppcmast <config_file>\n");
    exit(1);
  } 

  read_config_file(argv[1]);

  /* Inicialization of Q processes */
  contt = 0;
  cont  = 0; 
  qtpp  = div(n,nproc);

  for (i=0;i<nproc;i++){
    contt = pvm_spawn(Q,              /* executable file name               */
		      NULL,           /* no list of arguments               */
		      PvmTaskDefault, /* process can be started in any node */
		      /*maquinas[i]*/NULL,         /* machine used                  */
		      qtpp.quot,           /* number of copies of the task  */
		      &neig[i*qtpp.quot]); /* process identifier            */
    if(contt!=qtpp.quot){
      printf ("ERROR: Can only start %d q proccesses (%s)!\n",contt, Q);
      pvm_exit();
      exit(0);
    }
    cont+=contt;
  }

  /* Sending Neighborhood */
  for (i=0; i<n; i++){
    int value;
    pvm_initsend(PvmDataDefault);
    pvm_pkint(&cont,1,1);
    pvm_pkint(neig,n,1);
    for (k=0;k<n;k++){
      pvm_pkint(Delta[k],n,1);
    }    
    pvm_pkstr(PFile);
    pvm_pkstr(ParamFile);
    value = rand()+neig[i];
    pvm_pkint(&value, 1,1);
    pvm_send(neig[i],0);
    printf ("Neighborhood sent to %x\n",neig[i]);
  }

  pvm_exit();
  exit(0);
}


/* read the configuration of nodes, stored in CFile */
int read_config_file(char *CFile) {
  int  i,j,deltaij;
  FILE *f;

  if((f=fopen(CFile,"rb"))==NULL) {
    printf("ERROR: I cannot open file %s\n", CFile);
    exit(1);
  }

  fscanf(f,"Number of Processors = %d\n", &nproc);
  fscanf(f,"Number of Nodes = %d\n", &n);
  fscanf(f,"Name of Executable File P = %s\n", PFile);
  fscanf(f,"Parameter File = %s\n", ParamFile);
  fscanf(f,"Delta[%d][%d]=%d\n", &i, &j, &deltaij);
  Delta[i][j]=deltaij;
 
  while ((j != 0) || (i != 0)){
    fscanf(f,"Delta[%d][%d]=%d\n", &i, &j, &deltaij);
    Delta[i][j]=deltaij;
  }

  fclose(f);
  return(0);   
}
