/*******************************************************
 *	PPC program - window where graphic will be draw
 * 
 *	(C) 1998-2001, Carlos A.S. Oliveira
 *******************************************************/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "ppc.h"
#include "ateams.h"

/*  -----------------------------
 *  Data structures and defines
 *  ----------------------------- */

void run_ateams(Graph *G, vert s[], vert d[], int p);

/* data structure for graph nodes on screen */
typedef struct _GraphNode {
  int x, y;                /* window position */
  int type;                /* vertex type */
} GraphNode;

#define VERT_SOURCE	1
#define VERT_DEST	2
#define VERT_NOSD	3

/* half width of a graph node */
#define NODE_HALFSIZE	4

/* true if point (x, y) is inside the vertex i */
#define inside_node(i, x, y)                   \
	(node_xpos(i)-NODE_HALFSIZE <= (x)     \
	 && (x) <= node_xpos(i)+NODE_HALFSIZE  \
	 && node_ypos(i)-NODE_HALFSIZE <= (y)  \
	 && (y) <= node_ypos(i)+NODE_HALFSIZE)

/* true if point (x, y) is inside the rectangle r */
#define inside_rect(r, x, y)                   \
	(r.left <= (x) && (x) <= r.right  \
	 && r.top <= (y) && (y) <= r.bottom)

#define node_xpos(i) (gpGNode[(i)].x + h_displacement)
#define node_ypos(i) (gpGNode[(i)].y + v_displacement)

#define vscreen_to_internal(y) (y - v_displacement);
#define hscreen_to_internal(x) (x - h_displacement);

#define create_rect(r,x1,y1,x2,y2) do { \
  r.left=x1; r.top=y1; r.right=x2; r.bottom=y2; } while (0)

#define HANDLE_MESSAGE(mess, func) \
  case mess: func; break

/*  -----------------------------
 *  Global variables
 *  ----------------------------- */

/*
 * extern variables
 */
/* program instance */
extern HINSTANCE ghInst;

/* current execution mode */
extern int gMode;

/* Define uma memo'ria global de solucoes */
extern raiz *solucoes;

/* ponteiro para vetor de nos do grafo */
static GraphNode *gpGNode = 0;

/* grafo por lista de adjacencias */
static Graph *gGrafo = 0;

/* number of vertices in the graph */
static int gNumVerts = 0;

/* nao zero se a janela ja' tiver sido criada (evita
   um erro no caso da criacao de mais de uma janela 
   dessa classe. Se mais de uma janela for criada, ela
   possuira' o mesmo conteu'do */
static int gCreated = 0;

/* nonzero if a-teams is running */
int gATRunning = 0;

/* mutex for thread synchronization */
HANDLE gMutex, gMutexStop;

/* handle for ateams thread */
static HANDLE gAteamsThreadHandle;

/* windows class name */
static char *strWClass = "PPCWindow";

/* default colors  */
static COLORREF gClSource, gClDest, gClLine,
  gClNode, gClBack, gClSol;

/* max size of the working area  and
   displacement inside this working area. */
static int max_vsize = 0, v_displacement = 0;
static int max_hsize = 0, h_displacement = 0;

static int gClicked_node = 0;
/* initial position during a drag node action */
static int gDrag_xpos = 0, gDrag_ypos = 0;

/* true if there is a selection rectangle */
static int gSelection = 0;

/* true if the selection is being dragged */
static int gDragSelection = 0;
static int gDrawing_selection = 0;

/* rectangle created by a selection */
static RECT gSel_rect;

/* starting point for dragging of selection  */
static POINT gDrag_sel_point;
static POINT gDrag_start_sel_point;

/*  -----------------------------
 *  Function prototypes
 *  ----------------------------- */

LRESULT CALLBACK 
PPCWindProc(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp);

static void
ResizeWindow(HWND hWnd);

static int
InitPPCWindow(HWND hWnd, LPARAM lp);

static void
create_graph_setup(HWND hWnd);

static int 
CreateWindowGraph(int nVert, int x, int y);

static int 
CreateEmptyGraph(int nVert);

static void
Paint(HWND hWnd);

static void
ButtonDown(HWND hWnd, WPARAM wp, LPARAM lp);

static void
ButtonUp(HWND hWnd, WPARAM wp, LPARAM lp);

static void
MouseMove(HWND hWnd, int px, int py);

static void
move_selection(HWND hWnd, int movx, int movy);

static void
draw_selection(HDC dc, RECT *r);

static int
Execute();

DWORD 
ThreadFunc(LPDWORD lpdwParam);

static void
Destroy();

static int
StopAteams();

static int 
ChangeGraph(HWND hWnd, WPARAM nVert);

static void
ChangeColor(HWND hWnd, WPARAM wp);

void SetDefaultColors();

static void
Scroll(HWND hWnd, WPARAM wp, LPARAM lp, int horz);

static void
SelMode_ButtonDown(HWND hWnd, int px, int py);

static void
DrawSDMode_ButtonDown(HWND hWnd, int px, int py);

static void
Save_graph(HWND hWnd, FILE *f);

static void
Read_graph(HWND hWnd, FILE *f);

static void
ErrorMsg(char *mes);

static int
EraseBackground(HWND hWnd, WPARAM wp);

static void
finish_drag_selection(HWND hWnd, int px, int py);

static void
finish_drawing_selection(HWND hWnd, int px, int py);

static void
finish_drag_vertex(HWND hWnd, int px, int py);

/*  -----------------------------
 *  Implementation of functions
 *  ----------------------------- */

/* 
 *  FUNCTION: CreatePPCW
 *  PURPOSE:  procedimento da janela que contem o grafo
 *  PARAMETERS:
 *  RETURN:
 *	padroes para um procedimento de janela
 */
LRESULT CALLBACK 
PPCWindProc(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp)
{
  switch (mess){
    HANDLE_MESSAGE(WM_CREATE, return InitPPCWindow(hWnd, lp));
    HANDLE_MESSAGE(WM_DESTROY, Destroy());
    HANDLE_MESSAGE(WM_LBUTTONUP, ButtonUp(hWnd, wp, lp));
    HANDLE_MESSAGE(WM_LBUTTONDOWN, ButtonDown(hWnd, wp, lp));
    HANDLE_MESSAGE(WM_MOUSEMOVE, MouseMove(hWnd, LOWORD(lp), HIWORD(lp)));
    HANDLE_MESSAGE(WM_SIZE, ResizeWindow(hWnd));
    HANDLE_MESSAGE(WM_EXECUTE, Execute());
    HANDLE_MESSAGE(WM_STOPATEAMS, return StopAteams());
    HANDLE_MESSAGE(WM_CHANGEGRAPH, return ChangeGraph(hWnd, wp));
    HANDLE_MESSAGE(WM_VSCROLL, Scroll(hWnd, wp, lp, 0));
    HANDLE_MESSAGE(WM_HSCROLL, Scroll(hWnd, wp, lp, 1));
    HANDLE_MESSAGE(WM_PAINT, Paint(hWnd));
    HANDLE_MESSAGE(WM_ERASEBKGND, return EraseBackground(hWnd, wp));
    HANDLE_MESSAGE(WM_SAVEYOURSELF, Save_graph(hWnd, (FILE*)lp));
    HANDLE_MESSAGE(WM_READGRAPH, Read_graph(hWnd, (FILE*)lp));    
  case WM_COMMAND:
    switch(wp){
    case IDM_COLOR_SOURCE:
    case IDM_COLOR_DEST:
    case IDM_COLOR_NODE:
    case IDM_COLOR_LINE:
    case IDM_COLOR_SOL:
    case IDM_COLOR_BACK:
      ChangeColor(hWnd, wp);
      break;
    case IDM_COLOR_DEFAULT:
      SetDefaultColors();
      InvalidateRect(hWnd, NULL, TRUE);		
    }
    break;
  default:
    return DefWindowProc(hWnd, mess, wp, lp);
  }
  return 0;        
}

/* basic step size for scrolling */
#define SB_STEP 5

/* 
 *  FUNCTION: Scroll
 *  PURPOSE: handle WM_HSCROLL and WM_HSCROLL messages
 *  PARAMETERS:
 */
void
Scroll(HWND hWnd, WPARAM wp, LPARAM lp, int horz)
{
  int code = (int) LOWORD(wp);
  short int spos = (short int) HIWORD(wp);
  static int hpos = 0;
  static int vpos = 0;  
  int move = 0;
  RECT r;
  switch (code){
  case SB_BOTTOM:
    if (horz)
      hpos = max_hsize;
    else
      vpos = max_vsize;
    break;

  case SB_TOP:
    if (horz) hpos = 0;
    else vpos = 0;
    break;

  case SB_LINEUP:
    if (horz) {
      if (hpos > 0) {
	hpos--; move--;
      }
    } else {
      if (vpos > 0) {
	vpos--; move--;
      }
    }
    break;

  case SB_LINEDOWN:
    if (horz) {
      if (hpos >= max_hsize) {
	SetScrollRange(hWnd, SB_HORZ, 0, ++max_hsize, TRUE);
      }
      hpos++; move++;
    } else {
      if (vpos >= max_vsize) {
	SetScrollRange(hWnd, SB_VERT, 0, ++max_vsize, TRUE);
      }
      vpos++; move++;
    }
    break;

    /* used when clicking in page down or up */
  case SB_PAGEUP:
  case SB_PAGEDOWN:
    {
      int i;
      for (i=0; i<SB_STEP; i++)
	SendMessage(hWnd, (horz? WM_HSCROLL : WM_VSCROLL),
		    (code == SB_PAGEUP? SB_LINEUP : SB_LINEDOWN), 0L);
    }
    break;

    /* used when moving the scroll */
  case SB_THUMBTRACK:
    if (horz) {
      move = spos - hpos;
      hpos = spos;
    } else {
      move = spos - vpos;
      vpos = spos;
    }
    break;
  }
  GetClientRect(hWnd, &r);
  if (horz)
    h_displacement += (move*SB_STEP);
  else
    v_displacement += (move*SB_STEP);
  SetScrollPos(hWnd, (horz ? SB_HORZ : SB_VERT),
	       (horz ? hpos : vpos), TRUE);
  ScrollWindowEx(hWnd, (horz ? move*SB_STEP : 0),
		 (horz ? 0 : move*SB_STEP),
		 0, 0, 0, 0, SW_INVALIDATE);
  /* this is necessary to redraw the
     small area scrolled from */
  r.left = r.right - SB_STEP;
  r.top = r.bottom - SB_STEP;
  InvalidateRect(hWnd, &r, TRUE);
  UpdateWindow(hWnd);
}

static void
Save_graph(HWND hWnd, FILE *f)
{
  int i;
  /* print num of vertices and arests */
  fprintf(f, "n %d\nm %d\n", gNumVerts, gr_narests(gGrafo));
  /* print arests */
  for (i=1; i<=gNumVerts; i++) {
    graph_node *no, *h;
    no = gr_get_adj_list(gGrafo, i);
    if (no) {
      h = no;
      for (no=no->prox; no; no=no->prox)
	if (i < no->v) 
	  fprintf(f, "a %d %d %d\n", i, no->v,
		 gr_get_cost(gGrafo, i, no->v));
    }
  }
  /* print displacements */
  fprintf(f, "# %d %d\n", h_displacement, v_displacement);
  for (i=1; i<=gNumVerts; i++) {
    /* print position */
    fprintf(f, "# %d %d %d\n", gpGNode[i].x, gpGNode[i].y, gpGNode[i].type);
  }
  /* indicates end of saving */
  SendMessage(GetParent(hWnd), WM_END_FILE_OPERATION, 0, 0);
}

static void
Read_graph(HWND hWnd, FILE *f)
{
  RECT r;
  int i, m, n, v1, v2, c;
  /* destroy previous graph */
  Destroy(hWnd);
  /* read initial information */
  fscanf(f, "n %d\n", &gNumVerts);
  if (gNumVerts < 2){
    ErrorMsg("Wrong number of vertices");
    return;
  }
  n = gNumVerts;
  GetClientRect(hWnd, &r);
  /* inicializa o randomizador */
  if (CreateEmptyGraph(n) != -1) {
    fscanf(f, "m %d\n", &m);
    if (m < 0){
      ErrorMsg("Wrong number of vertices");
      return;
    }
    /* read arests */
    for (i=1; i<=m; i++) {
      fscanf(f, "a %d %d %d\n", &v1, &v2, &c);
      if (v1 < 1 || v2 < 1 || c < 1 ||
	  v1 > n || v2 > n){
	char err[256];
	sprintf(err, "Wrong values for arest. "
		"i=%d, %d, %d, %d", i, v1, v2, c);
	ErrorMsg(err);
	return;
      }
      gr_set_arest(gGrafo, v1, v2, c);
    }
    /* read displacements */
    fscanf(f, "# %d %d\n", &h_displacement, &v_displacement);
    /* read positions */
    for (i=1; i<=n; i++) {
      fscanf(f, "# %d %d %d\n",
	     &gpGNode[i].x, &gpGNode[i].y, &gpGNode[i].type);
    }
  } else {
    ErrorMsg("Can not create graph");
  }
  create_graph_setup(hWnd);
  /* report the end of reading operation */
  SendMessage(GetParent(hWnd), WM_END_FILE_OPERATION, 0, 0);
  /* redraw the window */
  InvalidateRect(hWnd, 0, TRUE);
  UpdateWindow(hWnd);
}

/* 
 *  FUNCTION: RegisterPPCW
 *  PURPOSE: registra a janela do grafo. 
 *  PARAMETERS:
 */
void
RegisterPPCW()
{
  WNDCLASS wc;
  memset(&wc, 0, sizeof(wc));
  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = PPCWindProc;
  wc.hInstance = (HWND)ghInst;
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.lpszClassName = strWClass;
  RegisterClass(&wc);
}

/* 
 *  FUNCTION: CreatePPCW
 *  PURPOSE:  cria a janela do grafo
 *  PARAMETERS:
 *	parent - janela pai 
 *	x1, y1, x2, y2 - posicao e tamanho iniciais
 *	numVerts - numero de vertices no grafo
 */
HWND 
CreatePPCW(HWND parent, int x1, int y1, int x2, int y2, int numVerts)
{
  HWND h;
  h = CreateWindowEx(WS_EX_CLIENTEDGE, strWClass, "", 
		     WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_HSCROLL,
		     x1, y1, x2,y2, 
		     parent, NULL, ghInst, (LPVOID) numVerts);
  create_graph_setup(h);
  return h;
}

/* 
 *  FUNCTION: create_graph_setup
 *  PURPOSE:  Used everytime we need to create a new graph
 *  PARAMETERS:
 *	hWnd - window handle
 */
void
create_graph_setup(HWND hWnd)
{
    RECT r;
    int x1, x2, y1, y2;
    GetClientRect(hWnd, &r);
    x1 = r.left; x2 = r.right;
    y1 = r.top;  y2 = r.bottom;
    max_vsize = (y2-y1)/SB_STEP;
    SetScrollRange(hWnd, SB_VERT, 0, max_vsize, FALSE);
    max_hsize = (x2-x1)/SB_STEP;
    SetScrollRange(hWnd, SB_HORZ, 0, max_hsize, FALSE);
    h_displacement = v_displacement = 0;
}

/* 
 *  FUNCTION: ResizeWindow
 *  PURPOSE:  redimensiona a janela  
 *  PARAMETERS:
 *	hWnd - janela pai 
 */
void
ResizeWindow(HWND hWnd)
{
}

static int
EraseBackground(HWND hWnd, WPARAM wp)
{
  RECT rc;
  GetUpdateRect(hWnd, &rc, FALSE);
  FillRect((HDC)wp, &rc, CreateSolidBrush(gClBack));
  return 1;
}

/* 
 *  FUNCTION: InitPPCWindow
 *  PURPOSE: faz as inicializacoes necessarias para a janela
 *	do grafo. 
 *  PARAMETERS:
 *	hWnd - handle da janela
 *	lp   - parametro enviado com a mensagem WM_CREATE
 *  RETURN:	
 *	-1 se houve problemas, 0 caso contrario
 */
int
InitPPCWindow(HWND hWnd, LPARAM lp)
{
  gCreated++;
  if (gCreated == 1){
    CREATESTRUCT *cs = (LPCREATESTRUCT) lp;
    int nVert = (int) cs->lpCreateParams;
    RECT r;
    char *erro = "Numero de ns invalido";
	
    if (nVert < 2){
      ErrorMsg(erro);
      return -1;
    }
    /* inializa as cores de desenho */
    SetDefaultColors();
    gNumVerts = nVert;
    /* cria dois mutex para sincronizacao dos threads */
    gMutex = CreateMutex(NULL, FALSE, NULL);
    gMutexStop = CreateMutex(NULL, FALSE, NULL);
    GetClientRect(hWnd, &r);
    /* inicializa o randomizador */
    srand((unsigned)clock());
    return CreateWindowGraph(nVert, r.right - r.left, r.bottom - r.top);
  } else {
    return 0;
  }
}

/* 
 *  FUNCTION: CreateWindowGraph
 *  PURPOSE: Manipula a mensagem de pintura da janela 
 *  PARAMETERS:
 *	nVert - numero de vertices do grafo
 *	x, y  - tamanho da janela na qual ele sera desenhado
 *  RETURN:	
 *	-1 se houve problemas, 0 caso contrario
 */
int 
CreateWindowGraph(int nVert, int x, int y)
{
  int i, rx, ry;
  /* inicializa o vetor de nos */
  gpGNode = NewVec(GraphNode, (nVert+1));
  if (!gpGNode) 
    goto erro1;
  gGrafo  = gr_create(nVert);
  if (!gGrafo) 
    goto erro2;
		
  rx = (x - 50 > 50) ? x - 50 : 50;
  ry = (y - 50 > 50) ? y - 50 : 50;
  /* gera os nos do grafo de forma aleatoria */
  for (i=1; i<=nVert; i++){
    gpGNode[i].x = (rand() % rx)+ 25;
    gpGNode[i].y = (rand() % ry)+ 25;
    gpGNode[i].type = VERT_NOSD;
  }

  /* Gera as arestas do grafo */
  for (i=1; i<=nVert; i++){
    int j;
    for (j=i+1; j<=nVert; j++){
      if (rand()%3 == 1){     /* 30 por cento de chance */
	int x1 = gpGNode[i].x, y1 = gpGNode[i].y;
	int x2 = gpGNode[j].x, y2 = gpGNode[j].y;
	/* distance between points i,j */
	int dist = (int) 
	  sqrt((float)((x1*x1 - 2*x1*x2 + x2*x2) 
		       + (y1*y1 - 2*y1*y2 + y2*y2))); 
	gr_set_arest(gGrafo, i, j, dist);
      }		
    }
  }

  /* preenche a estrutura instancia do PPC */
  /* the PPC instance */
  gInst = New(ppc_instance);
  if (!gInst) 
    goto erro3;

  gInst->source = NewVec(vert, nVert);
  if (!gInst->source)
    goto erro4;

  gInst->dest = NewVec(vert, nVert);
  if (!gInst->dest)
    goto erro5;

  gInst->hasBound = 0;
  strcpy(gInst->instName, "");
  gInst->G = gGrafo;
  return 0;

 erro5:
  Delete(gInst->source);
 erro4:
  Delete(gInst);
 erro3:
  gr_delete(gGrafo);
 erro2:
  Delete(gpGNode);
 erro1:
  return -1;

}

int 
CreateEmptyGraph(int nVert)
{
  /* inicializa o vetor de nos */
  gpGNode = NewVec(GraphNode, (nVert+1));
  if (!gpGNode) 
    goto erro1;
  gGrafo  = gr_create(nVert);
  if (!gGrafo) 
    goto erro2;
		
  /* preenche a estrutura instancia do PPC */
  /* the PPC instance */
  gInst = New(ppc_instance);
  if (!gInst) 
    goto erro3;

  gInst->source = NewVec(vert, nVert);
  if (!gInst->source)
    goto erro4;

  gInst->dest = NewVec(vert, nVert);
  if (!gInst->dest)
    goto erro5;

  gInst->hasBound = 0;
  strcpy(gInst->instName, "");
  gInst->G = gGrafo;
  return 0;
 erro5:
  Delete(gInst->source);
 erro4:
  Delete(gInst);
 erro3:
  gr_delete(gGrafo);
 erro2:
  Delete(gpGNode);
 erro1:
  return -1;

}


/* 
 *  FUNCTION: Paint
 *  PURPOSE: Manipula a mensagem de pintura da janela 
 *  PARAMETERS:
 *	hWnd - janela pai 
 */
static void
Paint(HWND hWnd)
{
  int i, update = 0;
  static int selection_recently_used = 0;
  HDC dc;
  RECT rc;
  PAINTSTRUCT p;
  HBRUSH source, dest, line, vertex, old;
  HPEN pen;
  Solution  *sol;

  if (!GetUpdateRect(hWnd, NULL, TRUE))
    return;
  
  source = CreateSolidBrush(gClSource); 
  dest = CreateSolidBrush(gClDest);
  line = CreateSolidBrush(gClLine);
  vertex = CreateSolidBrush(gClNode);
  BeginPaint(hWnd, &p);
  dc = p.hdc;

  if (!gSelection || selection_recently_used) {
    /* correctly erases the last selection,
       because the last time this was make with ROP2, and we
       still have lines in the background */
    draw_selection(dc, &gSel_rect);
    selection_recently_used = 0;
  }

  /*old = */SelectObject(dc, line);
  pen = CreatePen(PS_SOLID, 1, gClLine);
  old = SelectObject(dc, pen);
  
  /* draw the edges first */ 
  for (i=1; i<=gNumVerts; i++){
    graph_node *h, *no;
    no = gr_get_adj_list(gGrafo, i); 
    if (!no )
      continue;
    h = no;
    for (no=no->prox; no; no=no->prox){
      MoveToEx(dc, node_xpos(i), node_ypos(i), NULL);
      LineTo(dc, node_xpos(no->v), node_ypos(no->v));
    }
    while (h) { no = h->prox; delete_graph_node(h); h = no; }
  }
  SelectObject(dc,old);
  DeleteObject(pen);

  WaitForSingleObject(gMutex, INFINITE);
  if (gATRunning){
    sol = copy_solution(solucoes[best_solution_pos].sol, gInst->p);
    update = 1;
  }
  if (!ReleaseMutex(gMutex))
    ErrorMsg("Mutex erro em Paint");

  /* desenha a solucao atual, se necessario */
  if (update){
    pen = CreatePen(PS_SOLID, 2, gClSol);
    old = SelectObject(dc, pen);
    for (i=0; i<gInst->p; i++){
      listvert *no = sol[i].path;
      while (no->prox){
	MoveToEx(dc, node_xpos(no->v), node_ypos(no->v), NULL);
	LineTo(dc, node_xpos(no->prox->v), node_ypos(no->prox->v));
	no = no->prox;
      }
    }
    SelectObject(dc,old);
    DeleteObject(pen);
    DeleteSolution(sol, gInst->p);
  }		

  /* desenha cada vertice do grafo */
  for (i=1; i<=gNumVerts; i++){
    HBRUSH b = gpGNode[i].type == VERT_SOURCE ? 
      source  : (gpGNode[i].type == VERT_DEST ? dest : vertex);

    SetRect(&rc, node_xpos(i)-NODE_HALFSIZE, node_ypos(i)-NODE_HALFSIZE, 
	    node_xpos(i)+NODE_HALFSIZE, node_ypos(i)+NODE_HALFSIZE);
    FillRect(dc, &rc, b);
  }

  /* draw the selection rectangle */
  if (gSelection) {
    int mode =  SetROP2(dc, R2_NOTXORPEN);
    draw_selection(dc, &gSel_rect);
    SetROP2(dc, mode);
    selection_recently_used = 1;
  } 
  DeleteObject(line);
  DeleteObject(source);
  DeleteObject(dest);
  DeleteObject(vertex);
  EndPaint(hWnd, &p);
}

void finish_drag_selection(HWND hWnd, int px, int py)
{
  /*graph_node *no;*/
  int i, incx, incy;
  RECT orig_rect;
  move_selection(hWnd,
		 gDrag_sel_point.x - px,
		 gDrag_sel_point.y - py);
  gDragSelection = 0;
  /* move all points inside the selection to
     this new location. */
  incx = gDrag_start_sel_point.x - px;
  incy = gDrag_start_sel_point.y - py;
  create_rect(orig_rect, gSel_rect.left+incx, gSel_rect.top+incy,
	      gSel_rect.right+incx, gSel_rect.bottom+incy);
  for (i=1; i<=gNumVerts; i++) {
    if (inside_rect(orig_rect, node_xpos(i), node_ypos(i))) {
      gpGNode[i].x -= incx;
      gpGNode[i].y -= incy;
      /* FIXME: this can be improved by finding
	 only what need to be redraw ... */
      InvalidateRect(hWnd, 0, TRUE);
    }
  }
}

void finish_drag_vertex(HWND hWnd, int px, int py)
{
  int i = gClicked_node, maxx, minx, maxy, miny;
  if ((gDrag_xpos != px) || (gDrag_ypos != py)) {
    graph_node *h, *no;
    int oldx = node_xpos(i), oldy = node_ypos(i);
    RECT r;
    /* update coordinates of the graph node */
    gpGNode[i].x = hscreen_to_internal(px);
    gpGNode[i].y = vscreen_to_internal(py);
    /* find min and max positions between new and old coordinates */
    minx = maxx = node_xpos(i);
    miny = maxy = node_ypos(i);
    if (oldx < minx) minx = oldx;
    else if (oldx > maxx) maxx = oldx;
    if (oldy < miny) miny = oldy;
    else if (oldy > maxy) maxy = oldy;
    /* find max and min positions among the adjacent vertices */
    no = gr_get_adj_list(gGrafo, i);
    if (no){
      h = no;
      for (no=no->prox; no; no=no->prox){
	/* update max and min x pos */
	if (node_xpos(no->v) < minx)       minx = node_xpos(no->v);
	else if (node_xpos(no->v) > maxx)  maxx = node_xpos(no->v);
	/* update max and min y pos */
	if (node_ypos(no->v) < miny)       miny = node_ypos(no->v);
	else if (node_ypos(no->v) > maxy)  maxy = node_ypos(no->v);
      }
      /* delete adjacency list */
      while (h) { no = h->prox; delete_graph_node(h); h = no; }
    }
    /* now update this region of the screen */
    create_rect(r, minx - NODE_HALFSIZE, miny - NODE_HALFSIZE,
		maxx + NODE_HALFSIZE, maxy + NODE_HALFSIZE);
    InvalidateRect(hWnd, &r, TRUE);
    UpdateWindow(hWnd);
  }
}

void finish_drawing_selection(HWND hWnd, int px, int py)
{
  RECT r;
  if ((gDrag_xpos != px) || (gDrag_ypos != py)) {
    int x1 = (gDrag_xpos < px) ? gDrag_xpos : px;
    int y1 = (gDrag_ypos < py) ? gDrag_ypos : py;	
    int x2 = (gDrag_xpos >= px) ? gDrag_xpos : px;
    int y2 = (gDrag_ypos >= py) ? gDrag_ypos : py;	
    gSelection = 1;
    create_rect(gSel_rect, x1, y1, x2, y2);
    create_rect(r, x1, y1, x2+1, y2+1);
    InvalidateRect(hWnd, &r, FALSE);
    UpdateWindow(hWnd); 
  }
  gDrawing_selection = 0;
}

/* 
 *  FUNCTION: ButtonUp
 *  PURPOSE:  Handles mouse button up messages
 *  PARAMETERS:
 *	hWnd - handle of the window
 *      wp - parameter sent by Windows
 *      lp - parameter sent by Windows
 */
static void
ButtonUp(HWND hWnd, WPARAM wp, LPARAM lp)
{
  int px = LOWORD(lp), py = HIWORD(lp);
  if (gDragSelection) {
    finish_drag_selection(hWnd, px, py);
  } else if (gMode == STAT_SELECT) {
    if (gClicked_node != 0) {
      /* then we are dragging a vertex */
      finish_drag_vertex(hWnd, px, py);
    } else if (gDrawing_selection) {
      /* in this case we are finishing the
	 selection rectangle */
      finish_drawing_selection(hWnd, px, py);
    }
  }
  gClicked_node = 0;
}

/* 
 *  FUNCTION: MouseMove
 *  PURPOSE:  handle movements of the mouse
 *  PARAMETERS:
 *	hWnd - handle of the window
 *      px, py - coordinates x and y of the point
 */
void
MouseMove(HWND hWnd, int px, int py)
{
  if (gDragSelection) {
    move_selection(hWnd,
		   gDrag_sel_point.x - px,
		   gDrag_sel_point.y - py);
    gDrag_sel_point.x = px;
    gDrag_sel_point.y = py;
  }
}

void move_selection(HWND hWnd, int movx, int movy)
{
  HDC dc;
  HPEN pen, old;
  dc = GetDC(hWnd);
  /*pen = CreatePen(PS_SOLID, 1, gClSol);
    old = SelectObject(dc, pen);*/
  /* XOR draw mode */
  SetROP2(dc, R2_NOTXORPEN);
  /* this effectivelly erases the old shape */
  draw_selection(dc, &gSel_rect);
  gSel_rect.left -=  movx;
  gSel_rect.right -=  movx;
  gSel_rect.top -=  movy;
  gSel_rect.bottom -=  movy;
  /* Draw it again */    
  draw_selection(dc, &gSel_rect);
  SelectObject(dc, old);
  DeleteObject(pen);
  ReleaseDC(hWnd, dc);
}

void draw_selection(HDC dc, RECT *r)
{
  int x1 = r->left, y1 = r->top;
  int x2 = r->right, y2 = r->bottom;
  HPEN pen, old;
  POINT pts[5] =
    {{x1, y1}, {x2, y1}, {x2, y2}, {x1, y2},{x1, y1}};
  pen = CreatePen(PS_SOLID, 1, gClBack);
  old = SelectObject(dc, pen);
  Polyline(dc, pts, 5);
  SelectObject(dc,old);
  DeleteObject(pen);
}

/* 
 *  FUNCTION: ErrorMsg
 *  PURPOSE:  prints an error message
 *  PARAMETERS:
 *	mes - string with the message
 */
void
ErrorMsg(char *mes)
{
  MessageBox(0, mes, "", MB_OK | MB_ICONEXCLAMATION);
}


/* 
 *  FUNCTION: ButtonDown
 *  PURPOSE:  process the message WM_LBUTTONDOWN
 *  PARAMETERS:
 *	hWnd - handle of the window
 *      wp - parameter sent by Windows
 *      lp - parameter sent by Windows
 */
void
ButtonDown(HWND hWnd, WPARAM wp, LPARAM lp)
{
  switch (gMode){
  case STAT_SELECT:
    SelMode_ButtonDown(hWnd, LOWORD(lp), HIWORD(lp));
    break;
  case STAT_DRAW_SOURCE:
  case STAT_DRAW_DEST:
    DrawSDMode_ButtonDown(hWnd, LOWORD(lp), HIWORD(lp));
    break;
  }
}

/* 
 *  FUNCTION: SelMode_ButtonDown
 *  PURPOSE:  Executed when window clicked in
 *	``selection mode''.
 *  PARAMETERS:
 *	px - x coordinate
 *	py - y coordinate
 */
void
SelMode_ButtonDown(HWND hWnd, int px, int py)
{
  int i;
  RECT r;
  gClicked_node = 0;
  if (gSelection && inside_rect(gSel_rect, px, py)) {
    gDragSelection = 1;
    gDrag_start_sel_point.x = gDrag_sel_point.x = px;
    gDrag_start_sel_point.y = gDrag_sel_point.y = py;
  } else {
    for (i=1; i<=gNumVerts; i++){
      if (inside_node(i, px, py)) {
	gClicked_node = i;
	break;
      }
    }
    /* if there is still a selected rectangle */
    if (gSelection){
      gSelection = 0;
      create_rect(r, gSel_rect.left, gSel_rect.top,
		  gSel_rect.right+1, gSel_rect.bottom+1);
      InvalidateRect(hWnd, &r, FALSE);
      UpdateWindow(hWnd);
    }
    /* in this case we start to draw a selection rectangle */
    gDrag_xpos = px;
    gDrag_ypos = py;
    gDrawing_selection = 1;
  }
}

void
DrawSDMode_ButtonDown(HWND hWnd, int px, int py)
{
  HDC dc;
  RECT rc;
  HBRUSH b, old;
  int i;
  for (i=1; i<=gNumVerts; i++){
    if (inside_node(i, px, py)) {
      COLORREF color =
	(gMode == STAT_DRAW_SOURCE) ? gClSource : gClDest;
      dc = GetDC(hWnd);
      b = CreateSolidBrush(color);
      old = SelectObject(dc, b);

      SetRect(&rc, gpGNode[i].x-NODE_HALFSIZE, gpGNode[i].y-NODE_HALFSIZE, 
	      gpGNode[i].x+NODE_HALFSIZE, gpGNode[i].y+NODE_HALFSIZE);
      FillRect(dc, &rc, b);

      SelectObject(dc, old);
      DeleteObject(b);
      ReleaseDC(hWnd, dc);		
		
      gpGNode[i].type =
	(gMode == STAT_DRAW_SOURCE) ? VERT_SOURCE : VERT_DEST;
      break;
    }
  }
}

int
Execute()
{
  DWORD result; 
  int i;
  int ns = 0, nd = 0;
  
  for (i=1; i<=gNumVerts; i++){
    if (gpGNode[i].type == VERT_SOURCE){
      gInst->source[ns] = i;
      ns++;
    }	
    else if (gpGNode[i].type == VERT_DEST){
      gInst->dest[nd] = i;
      nd++;
    }
  }
  if ( ns != nd ){
    ErrorMsg("Erro: O nmero de fontes e destinos  diferente!");
    return 0;
  }
  if ( ns == 0 ){
    ErrorMsg("Erro: Voce precisa definir destinos e origens!");
    return 0;
  }
  
  gInst->p = ns;
  gStopAteams = 0;
  
  /* cria um novo thread que ira executar o A-Teams */
  gAteamsThreadHandle = CreateThread( 
		NULL,                        /* no security attributes        */ 
		0,                           /* use default stack size        */ 
		(LPTHREAD_START_ROUTINE) ThreadFunc, /* thread function       */ 
		NULL,                        /* no argument to thread function*/ 
		0,			           /* normal                        */ 
		&result);                    /* returns the thread identifier */ 
 
  /* Checa o valor de retorno  */  
  if (gAteamsThreadHandle == NULL) {
    ErrorMsg("Erro na execuo do A-Teams\n"); 
    return 0;
  }
  return 1;
}

 
/* 
 *  FUNCTION: ThreadFunc
 *  PURPOSE: passado como argumento para CreateThread. Inicia
 *	o thread de execucao do A-Teams. 
 *  PARAMETERS:
 *	param  - parametro passado pelo Windows.
 *  RETURN:	
 *	sempre 0
 */
DWORD 
ThreadFunc(LPDWORD param) 
{ 
	/* execute the A-Teams */
	run_ateams(gInst->G, gInst->source, gInst->dest, gInst->p);
	return 0; 
}

/* 
 *  FUNCTION: Destroy
 *  PURPOSE: Destroi a janela e limpa a memoria alocada 
 *  PARAMETERS:
 *  RETURN:	
 *	void
 */
void
Destroy()
{
  gCreated--;
  if (gCreated == 0) {
	Delete(gInst->source);
	Delete(gInst->dest);
	Delete(gInst);
	Delete(gpGNode);
	gr_delete(gGrafo);
  }
}

/* 
 *  FUNCTION: StopAteams
 *  PURPOSE: Stop the thread where the A-Team is executing
 *  PARAMETERS: none
 *  RETURN: void
 */
int StopAteams()
{
  WaitForSingleObject(gMutex, INFINITE);
  gStopAteams = 1;			
  ReleaseMutex(gMutex);
  return 1;
}

/* 
 *  FUNCTION: Change Graph
 *  PURPOSE: Erase the current graph and creates a new
 *	one with nVert vertices.
 *  PARAMETERS:
 *	hWnd - handle of the window
 *      nVert - number of vertices
 *  RETURN: 0 if sucessful
 */
int ChangeGraph(HWND hWnd, WPARAM nVert)
{
  RECT r;
  Destroy();
  if (nVert < 2){
    ErrorMsg("Numero de ns invalido");
    return -1;
  }
  gNumVerts = nVert;
  GetClientRect(hWnd, &r);
  if (CreateWindowGraph((int)nVert, r.right - r.left, r.bottom - r.top)== -1)
    return 0;
  create_graph_setup(hWnd);
  InvalidateRect(hWnd, NULL, TRUE);
  return 0;
}

/* 
 *  FUNCTION: ChangeColor
 *  PURPOSE: Calls a choose color dialog and change
 *	color for drawing objects
 *  PARAMETERS:
 *	hWnd - handle of the window
 *      wp - wparam parameter
 *  RETURN: void
 */
void
ChangeColor(HWND hWnd, WPARAM wp)
{
  CHOOSECOLOR cc;
  /* array of gray colors */
  COLORREF acrCustClr[16] = { 
    RGB(255, 255, 255), RGB(239, 239, 239), 
    RGB(223, 223, 223), RGB(207, 207, 207), 
    RGB(191, 191, 191), RGB(175, 175, 175), 
    RGB(159, 159, 159), RGB(143, 143, 143), 
    RGB(127, 127, 127), RGB(111, 111, 111), 
    RGB(95, 95, 95), RGB(79, 79, 79), 
    RGB(63, 63, 63), RGB(47, 47, 47), 
    RGB(31, 31, 31), RGB(15, 15, 15) 
  };

  cc.lStructSize = sizeof(CHOOSECOLOR); 
  cc.hwndOwner = hWnd; 
  cc.hInstance = (HWND)ghInst; 
  cc.lpCustColors = NULL; 
  cc.Flags = CC_FULLOPEN | CC_RGBINIT; 
  cc.lpCustColors = acrCustClr;
	 
  switch(wp){
  case IDM_COLOR_SOURCE:
    cc.rgbResult = gClSource; 
    break;
  case IDM_COLOR_DEST:
    cc.rgbResult = gClDest;
    break;
  case IDM_COLOR_NODE:
    cc.rgbResult = gClNode;
    break;
  case IDM_COLOR_LINE:
    cc.rgbResult = gClLine;
    break;
  case IDM_COLOR_SOL:
    cc.rgbResult = gClSol;
    break;
  case IDM_COLOR_BACK:
    cc.rgbResult = gClBack;
    break;
  default:
    return;
  }
  if ( !ChooseColor(&cc) )
    return;
  
  switch(wp){
  case IDM_COLOR_SOURCE:
    gClSource = cc.rgbResult; 
    break;
  case IDM_COLOR_DEST:
    gClDest = cc.rgbResult;
    break;
  case IDM_COLOR_NODE:
    gClNode = cc.rgbResult;
    break;
  case IDM_COLOR_LINE:
    gClLine = cc.rgbResult;
    break;
  case IDM_COLOR_SOL:
    gClSol = cc.rgbResult;
    break;
  case IDM_COLOR_BACK:
    gClBack = cc.rgbResult;
  }
  InvalidateRect(hWnd, NULL, TRUE);
}


/* 
 *  FUNCTION: ChangeColor
 *  PURPOSE: Called to set/restaure default colors.
 *  PARAMETERS: none
 *  RETURN: void
 */
void
SetDefaultColors()
{
  gClSource = RGB(0,128,0);
  gClDest = RGB(187,0,0);
  gClLine = RGB(255,255,172);
  gClNode = RGB(0,255,255);
  gClBack = RGB(0,0,0);
  gClSol = RGB(255,0,0);
}

void run_ateams(Graph *G, vert s[], vert d[], int p)
{
  /* Initialize data structures, open files, etc. */
  gNumSolutions = 2*gr_nverts(G);
  g_of = stdout;
  init_ateam(G, s, d, p);
  protect_thread;
  gBestSolValue  = solucoes[best_solution_pos].custo;
  gWorstSolValue = solucoes[worst_solution_pos].custo;
  gATRunning = 1;
  EnablePPCButton(IDM_GRAPH_STOP, TRUE);
  unprotect_thread;

  /* main loop -  run agents assyncronously */
  while (!gStopAteams){
    do_some_computation(G, s, d, p);
  }
}
