/*
  Sincq.c
  Partially Synchronous Model Synchronizer
*/


/* Message Labels

1. Neighborhood
2. Startup
3. Computation
4. Ack
5. Safe
99. Termination
*/

#include "pvm3.h"
#include <stdio.h>
#include <memory.h>

#define nmax 10
#define P "/u/calb/ppc/src/ateamp"
#define pos(a) (a)%(2*deltamax+2) 

#define TRUE 1
#define FALSE 0

#define OUT_FILE_NAME RELATIVE_PATH "q%x.txt"

/* Message Label type Definition */
typedef enum {
neigmsg,startup,compmsg,ack,safe,termination
}msglabel_t;

/* Compmsg type Definition */
typedef struct c{
   int origin;
   int pulse_orig;
   int size;
   char *contents;
   struct c *next;
}msg_t;

/* Pulse type definition */
typedef struct{
   int delaybound; /* Higher interval that the message set can be delayed */
   int *expected;
   int *safe;
   msg_t *firstmsg;
   msg_t *lastmsg;
} pulse_t;

void AddMsg (msg_t *msg, int si); 
int AllTrue(void); 
int DelayMsg(int si); 
void CleanPulse(int si);

pulse_t *s;

int n;
int currents, i;
int numneig;
int *neigp;
int *deltap;
int **delta;

int deltamax;
int oneinf;
int allinf;
FILE *outfile;


int last_slash_pos(char *str)
{
  int i, resp;
  for (i=0; *str; i++, str++)
    if (*str == '/')
      resp = i;
  return resp;
}

int main()
{
  /* Variables Needed */
  int mytid;
  int ptid;
  int *neig;
  int z;
  int trash;
  int contgo;
  int temp[1];
  int idp;
  int end;
  char PFile[80];
  /*char PFiletemp[80];*/
  char ParamFile[80], rel_path[80];
  char myhost[64];
  int /*count,*/ pos;
  int changed;
  msg_t *msgtmp;
  /* added by carlos */
  int rand_value; 


/* Inicialization */
  char OFName[20];

  mytid = pvm_mytid();
  ptid  = pvm_parent();


  /* Receiving of Neighborhood */
  n=0;
  trash=pvm_recv(ptid,0);
  pvm_upkint(&n,1,1);

  neig=(int *)calloc(n,sizeof(int));
  delta=(int **)calloc(n,sizeof(int *));
  for (z=0;z<n;z++)
    {
      delta[z]=(int *)calloc(n,sizeof(int));
    } /* for */
  neigp=(int *)calloc(n-1,sizeof(int));
  deltap=(int *)calloc(n-1,sizeof(int));

  pvm_upkint(neig,n,1);

  deltamax=-2;
  oneinf=FALSE;
  allinf=FALSE;
  for (z=0;z<n;z++)
    {
      int j;
  
      pvm_upkint(delta[z],n,1);
      j=0;
      while (j<n)
	{
	  if ((!oneinf)&&(delta[z][j]==-1))
	    oneinf=TRUE;
	  if ((deltamax < delta[z][j]) && (delta[z][j]!=0))
	    {
	      deltamax=delta[z][j];
	    }
	  j++;
	}
    }
  /*fprintf (outfile,"DELTAMAX = %d\n",deltamax);*/
  pvm_upkstr(PFile);
  pvm_upkstr(ParamFile);

/* added by carlos */ 
  pvm_upkint(&rand_value,1,1);
  pos = last_slash_pos(ParamFile);
  strncpy(rel_path, ParamFile, pos+1);
  rel_path[pos+1] = '\x0';
  sprintf(OFName, "%sq%x.txt", rel_path, mytid);
  printf("Tentando abrir %s\n",OFName);
  if((outfile=fopen(OFName,"w"))==NULL)
    { 
      printf("Can't open output file\n");
      pvm_exit();
      exit(1);
    }


  /*strncpy(PFile,PFiletemp,strlen(PFiletemp)-1);*/

  if (deltamax==-1)
    {
      deltamax=1;
      allinf=TRUE;
    }
  s=(pulse_t *) calloc(2*deltamax+2,sizeof(pulse_t));
  for (i=0;i<2*deltamax+2;i++) 
    {
      s[i].expected=(int *)calloc(n,sizeof(int));
      s[i].safe=(int *)calloc(n,sizeof(int));
    }

  /*fprintf (outfile,"Meus vizinhos sao : ");
for (z=0;z<n;z++)
   fprintf (outfile,"%x,",neig[z]);
fprintf(outfile,"\n");
fflush(outfile);*/
/* Which Node Am I? */
  i=0;
  while(neig[i]!=mytid)
    i++;

  gethostname(myhost,64);

  /* Send Startup to all Neighbors */
  for (z=0;z<n;z++)
    {
      /*  if (neig[z]!=mytid)
	  {*/
      pvm_initsend(PvmDataDefault);
      pvm_send(neig[z],1);
      /*  }*/
    } /* for */
  currents=0;
  contgo=0;
  end = FALSE;
  while (!end)
    {
      int j,bufid,len,who,sorig;
      msglabel_t type;
  
      /* Recebimento de Mensagens */
      bufid=pvm_recv(-1,-1);
      trash = pvm_bufinfo(bufid, &len, (int *) &type, &who);
      j=0;
      if (who!=idp)
	while ((neig[j]!=who)&&(j<n))
	  j++;

      switch(type){

	/* Mensagem de Startup */
      case startup:
        contgo=contgo+1;
        if (contgo==n)
	  {
	    /* P proccesses inicialization*/
           
	    trash=pvm_spawn(PFile,NULL,PvmTaskDefault,myhost,1,temp);
	    idp=temp[0]; 
	    /*fprintf (outfile," My P is %x\n",idp);
	      fflush(outfile);
	      if (trash!=1)
	      {
              fprintf(outfile,"ERROR: Impossible to create P proccess. Aborting...\n");
              end=TRUE;
	      }*/ /*if */
	    /*fflush(outfile);*/
	    if (trash==1)
	      {
		/* Send Neighborhood to P */
		for (z=0;z<n;z++)
		  {
		    if (delta[i][z]!=0)
		      {
			neigp[numneig]=z;
			deltap[numneig]=delta[i][z];
			numneig++;
		      }
		  } /* for */
		pvm_initsend(PvmDataDefault);
		pvm_pkint(&numneig,1,1);
		pvm_pkint(neigp,numneig,1);
		pvm_pkint(deltap,numneig,1);
		pvm_pkstr(ParamFile);
		pvm_pkint(&i,1,1);
		/* added by carlos */
		pvm_pkint(&rand_value,1,1);
		pvm_send(idp,startup); 

	      } /* if */
	  } /* if */
        break; 

	/* Computation Message */ 
      case compmsg:
        /* comp_msg originated from p */
        if (who==idp)
	  {
	    int dest, size;
	    char *contents;

	    pvm_upkint(&dest,1,1);
	    size=len-sizeof(int);
       
	    contents = (char *)calloc(size,sizeof(char));
	    pvm_upkbyte(contents,size,1); 
	    pvm_initsend(PvmDataDefault);
	    pvm_pkint(&currents,1,1); 
	    pvm_pkbyte(contents,size,1); 
	    pvm_send(neig[dest],compmsg); 
	    /*fprintf(outfile,"Comp de P para %x\n",neig[dest]);
	      fflush(outfile);*/
	    s[pos(currents)].expected[dest]++; 
	    /*fprintf(outfile,"Novo Expected[%d] do pulso %d= %d\n",dest,currents,s[pos(currents)].expected[dest]); */
	    pvm_initsend(PvmDataDefault);
	    pvm_send(idp,ack);
	    free(contents); 
	  } /* if */
        else
	  {
	    /*comp_msg originated from q */
          
	    msg_t *msgrcvd;
	    int greater_s;

	    /*fprintf(outfile,"Compmsg from Q %x\n",neig[j]);
	      fflush(outfile);*/

	    msgrcvd=(msg_t *)calloc(1,sizeof(msg_t));
	    /* Translating Nieg index to neigp index */
	    z=0;
	    while((neigp[z]!=j)&&(z<numneig))
	      z++;
	    msgrcvd->origin=z;

	    pvm_upkint(&msgrcvd->pulse_orig,1,1);
	    msgrcvd->size=len-sizeof(int);
	    msgrcvd->contents=(char *)calloc(msgrcvd->size,sizeof(char));
	    pvm_upkbyte(msgrcvd->contents,msgrcvd->size,1);

	    msgrcvd->next = NULL;

	    if ((currents > msgrcvd->pulse_orig)||(delta[i][neigp[msgrcvd->origin]]==-1))
	      greater_s=currents; 
	    else
	      greater_s=msgrcvd->pulse_orig;
	    AddMsg(msgrcvd,greater_s+1);

	    pvm_initsend(PvmDataDefault);
	    pvm_pkint(&msgrcvd->pulse_orig,1,1);
	    pvm_send(neig[j],ack); 
	  } /* else */
        break;
	/* Ack Message */
      case ack:

	pvm_upkint(&sorig,1,1);
	/*fprintf(outfile,"Ack[%d] from Q %x\n",sorig,neig[j]);
	  fflush(outfile);*/
	s[pos(sorig)].expected[j]--;
	/*fprintf(outfile,"Novo Expected[%d] do pulso %d= %d\n",j,sorig,s[pos(sorig)].expected[j]);*/
	if ((delta[i][j]!=-1)&&((s[pos(sorig)].expected[j]==0)&&(s[pos(sorig)].safe[i])))
          {
	    pvm_initsend(PvmDataDefault);
	    pvm_pkint(&sorig,1,1);
	    pvm_send(neig[j],safe);
	    /*fprintf(outfile,"Enviando Safe[%d] para Q %x\n",sorig,neig[j]);
	      fflush(outfile);*/
          } /* if */
        break;

	/* Safe Message */
      case safe:
	/* safe originated from p */
	if (who==idp)
          {
	    /*fprintf(outfile,"Safe from P\n");
	      fflush(outfile);*/
	    s[pos(currents)].safe[i]=TRUE;
	    changed=FALSE;
	    z=0;
	    if (!allinf)
	      {
		while ((AllTrue()) && (DelayMsg(currents)))
		  {
		    /*int k;*/
		    currents++;

		    /*for(k=0;k<2*deltamax+1;k++)
		      {
		      for(z=0;z<numneig;z++)
		      {
		      if (s[k].safe[neigp[z]])
		      fprintf(outfile,"T");
		      else
		      fprintf(outfile,"F");
		      }
		      fprintf(outfile,"\n");
		      }*/
		    CleanPulse(currents+deltamax+1);
		    changed=TRUE;
                 
		  } /* while */
	      }
            else
	      {
                /*int k;*/
		currents++;
                 
                /*for(k=0;k<2*deltamax+1;k++)
		  {
		  for(z=0;z<numneig;z++)
		  {
		  if (s[k].safe[neigp[z]])
		  fprintf(outfile,"T");
		  else
		  fprintf(outfile,"F");
		  }      
		  fprintf(outfile,"\n");
		  }*/
		CleanPulse(currents+deltamax+1);
		changed=TRUE;
	      }
	    if (changed)
	      {
                int z;
		msg_t *m;

                /*fprintf(outfile,"Alterando Currents devido a safe de P\n");
		  fflush(outfile);*/
                pvm_initsend(PvmTaskDefault); 
                pvm_pkint(&currents,1,1);
                
                msgtmp=s[pos(currents)].firstmsg;
  
		while (msgtmp != NULL)
		  {
		    pvm_pkint(&msgtmp->origin,1,1);
		    /*fprintf(outfile,"Origin %d .",msgtmp->origin);*/
		    pvm_pkint(&msgtmp->pulse_orig,1,1);
		    /*fprintf(outfile,"Pulse_Orig - %d .",msgtmp->pulse_orig);*/
		    pvm_pkbyte(msgtmp->contents,msgtmp->size,1);
		    /*fprintf(outfile,"Contents - %d .\n",msgtmp->contents);
		      fflush(outfile);*/
		    m=msgtmp->next;
		    free(msgtmp->contents);
		    free(msgtmp);
		    msgtmp = m;
		  } /* while */ 
		z=-1; /* Indicates the end of Comp. Messages Set */
		pvm_pkint(&z,1,1);

		pvm_send(idp,safe);
		/*fprintf(outfile,"Sending Safe[%d] to P\n",currents);
		  fflush(outfile);*/
		s[pos(currents)].firstmsg = NULL;
		trash=currents-1;
		for (z=0;z<numneig;z++)
		  {
		    if (delta[i][neigp[z]]!=-1)
		      {
			pvm_initsend(PvmDataDefault);
			pvm_pkint(&trash,1,1);
			pvm_send(neig[neigp[z]],safe);
		      }
		  } /* for */
		changed=FALSE;
		/*               free(msgtmp->contents);
				 free(msgtmp);*/
	      } /* if */
	    else
	      {
		/*fprintf(outfile,"Alltrue foi falso de primeira! com safe de P\n");
		  fflush(outfile);*/
		for (z=0;z<numneig;z++)
		  {
		    if ((s[pos(currents)].expected[neigp[z]]==0)&&(delta[i][neigp[z]]!=-1))
		      {
			pvm_initsend(PvmDataDefault);
			pvm_pkint(&currents,1,1);
			pvm_send(neig[neigp[z]],safe);
		      } /* if */
		  } /* for */
	      } /* else */
          } /* if */
	else
          {
            /* Safe originated from q */
            int srcvd;

            pvm_upkint(&srcvd,1,1);
            /*fprintf(outfile,"Safe[%d] from Q %x\n",srcvd,neig[j]);
	      fflush(outfile);*/
            if ((currents-deltamax) <0)
	      for(z=0;z<=srcvd;z++)
		s[pos(z)].safe[j]=TRUE;
            else
	      for(z=(currents-deltamax);z<=srcvd;z++)
		s[pos(z)].safe[j]=TRUE;

            if (s[pos(currents)].safe[i])
	      {
		if(!allinf)
		  {
		    while ((AllTrue()) && (DelayMsg(currents)))
		      {
			/*int k;*/
			currents++;

			/*for(k=0;k<2*deltamax+1;k++)
			  {
			  for(z=0;z<numneig;z++)
			  {  
			  if (s[k].safe[neigp[z]])
			  fprintf(outfile,"T");
			  else
			  fprintf(outfile,"F");
			  }                              
			  fprintf(outfile,"\n");
			  }*/
			CleanPulse(currents+deltamax+1);
			changed=TRUE;
		      } /* while */
		  }
		else
		  {
		    /*int k;*/
		    currents++;
                 
		    /*for(k=0;k<2*deltamax+1;k++)
		      {
		      for(z=0;z<numneig;z++)
		      {
		      if (s[k].safe[neigp[z]])
		      fprintf(outfile,"T");
		      else
		      fprintf(outfile,"F");
		      }      
		      fprintf(outfile,"\n");
		      }*/
		    CleanPulse(currents+deltamax+1);
		    changed=TRUE;

		  }
		if (changed)
		  {
		    int z;
		    msg_t *m;
		  
		    /*fprintf(outfile,"Alterando Currents devido a safe de Q\n");
		      fflush(outfile);*/
		    pvm_initsend(PvmTaskDefault);
		    pvm_pkint(&currents,1,1);
                  
		    msgtmp=s[pos(currents)].firstmsg;
		    while (msgtmp != NULL)
		      {
			pvm_pkint(&msgtmp->origin,1,1);
			/*fprintf(outfile,"Origin %d .",msgtmp->origin);*/
			pvm_pkint(&msgtmp->pulse_orig,1,1);
			/*fprintf(outfile,"Pulse_Orig - %d .",msgtmp->pulse_orig);*/
			pvm_pkbyte(msgtmp->contents,msgtmp->size,1);
			/*fprintf(outfile,"Contents - %d .\n",msgtmp->contents);
			  fflush(outfile);*/
			m=msgtmp->next;
			free(msgtmp->contents);
			free(msgtmp);
			msgtmp = m;
		      } /* while */
		    z=-1;
		    pvm_pkint(&z,1,1);

		    pvm_send(idp,safe);
		    /*fprintf(outfile,"Sending Safe[%d] to P\n",currents);
		      fflush(outfile);*/
		    s[pos(currents)].firstmsg = NULL;
		    trash=currents-1;
		    for (z=0;z<numneig;z++)
		      {
			if (delta[i][neigp[z]]!=-1)
			  {
			    pvm_initsend(PvmDataDefault);
			    pvm_pkint(&trash,1,1);
			    pvm_send(neig[neigp[z]],safe);
			  }
		      } /* for */
		    changed=FALSE;
		    /*free(msgtmp->contents);
		      free(msgtmp);*/
		  } /* if */
		else 
		  {
		    /*fprintf(outfile,"Alltrue foi falso de primeira! com safe de Q\n");
		      fflush(outfile);*/
		    for (z=0;z<numneig;z++)
		      {
			if ((s[pos(srcvd)].expected[neigp[z]]==0)&&(delta[i][neigp[z]]!=-1))
			  {
			    /*    pvm_initsend(PvmDataDefault);
				  pvm_pkint(&srcvd,1,1);
				  pvm_send(neig[neigp[z]],safe);*/
			  } /* if */
		      } /* for */
		  } /* else */
	      } /* if */
          } /* else */
        break;
	/* Termination Message */
      case termination: 
	end=TRUE;
        break;
      } /* switch */
      /*fprintf(outfile,"Fim de processamento de mensagem corrente\n");
	fflush(outfile);*/
    } /* while */

  free(s);
  for (z=0;z<n;z++)
    free(delta[z]);
  free(delta);
  free(neig);
  free(neigp);
  free(deltap);
  fclose(outfile);
  pvm_exit();
  exit(0);
} /* main */

/* Function that includes a message msg in the specified pulse si */
void AddMsg(msg_t *msg, int si)
{
   int msgdelay;
   if (s[pos(si)].lastmsg ==NULL)
   {
      s[pos(si)].lastmsg=msg;
      s[pos(si)].firstmsg=msg;
      s[pos(si)].delaybound=deltamax+1;
   }
   else
   {
      s[pos(si)].lastmsg->next = msg;
      s[pos(si)].lastmsg = msg;
   }
   s[pos(si)].lastmsg->next = NULL;
   if (allinf)
   {
        msgdelay=0;
        /*fprintf (outfile,"Allinf -> msgdelay=0\n");
        fflush(outfile);*/
   }
   else
   {
       if (delta[i][neigp[msg->origin]]==-1)
       {
          msgdelay=deltamax;
       } 
       else
       {
          msgdelay=delta[i][neigp[msg->origin]] - (si - msg->pulse_orig);
       }
   }
   /*fprintf (outfile,"Delta[i][%d] = %d. MsgDelay= %d\n",neigp[msg->origin],delta[i][neigp[msg->origin]],msgdelay); */
   if (s[pos(si)].delaybound > msgdelay)
   {
      s[pos(si)].delaybound = msgdelay;
   }
   /*fprintf(outfile," - delaybound[%d] = %d\n",si,s[pos(si)].delaybound);*/
} /* AddMsg */

/* Function that returns whether safe[j](max(0,(currents-delta[j])+1) is TRUE for all j, such that j is i's neighbor */
int AllTrue (void)
{
   int alltrue;
   int z;
   
   alltrue=TRUE;
   z=0;
   while ((alltrue) && (z<numneig))
   {
      int maxdelta;
      if ((delta[i][neigp[z]]!=-1)&&((currents-delta[i][neigp[z]]+1) >= 0))
      {
         maxdelta=currents-delta[i][neigp[z]]+1;
/*         fprintf(outfile,"Currents= %d. Safe[3] para %d = %d Maxdelta %d.\n",currents,s[pos(3)].safe[neigp[z]],neigp[z],maxdelta);
         fflush(outfile); */
         alltrue=alltrue && s[pos(maxdelta)].safe[neigp[z]];
      }
      z++;

   } /* while */
   /*if (alltrue)
      fprintf(outfile,"AllTrue = TRUE.\n");
   else
      fprintf(outfile,"AllTrue = FALSE.\n");
   fflush(outfile);*/
   return(alltrue);
} /* AllTrue */

/* Function that tries to postpone the message set and returns True if it can do so */
int DelayMsg(int si)
{
   /*fprintf(outfile,"Delaymsg called with %d\n",si);
   fflush(outfile);*/
   if (s[pos(si)].firstmsg==NULL)
   {
       /*fprintf(outfile," - DelayMsg - MSg set is empty. Ok to increase S\n");
       fflush(outfile);*/
       return(TRUE);
   }
   else
   {
      if (s[pos(si)].delaybound > 0)
      {
         /*fprintf(outfile," - DelayMsg - delaybound greater than 0. Ok to increase S\n");
       fflush(outfile);*/
         if (s[pos(si+1)].lastmsg==NULL)
         {
            s[pos(si+1)].firstmsg=s[pos(si)].firstmsg;
            s[pos(si+1)].lastmsg=s[pos(si)].firstmsg;
            s[pos(si+1)].delaybound=s[pos(si)].delaybound-1;
         }
         else 
         {
            /*fprintf(outfile,"else 1, Origin lastmsg(si+1): %d\n",
s[pos(si+1)].lastmsg->origin);
            fprintf(outfile,"else 1, Origin firstmsg(si): %d - Pos(si+1):%d\n", s[pos(si)].firstmsg->origin, pos(si+1));
            fflush(outfile);*/
   
            s[pos(si+1)].lastmsg->next=s[pos(si)].firstmsg; 
            /*fprintf(outfile,"else 2\n");
            fflush(outfile);*/
            if ((s[pos(si)].delaybound-1) < s[pos(si+1)].delaybound)
               s[pos(si+1)].delaybound=s[pos(si)].delaybound-1; 
         }
         s[pos(si)].firstmsg=NULL; 
         return(TRUE); 
      } /* if */
      else
      {
         /*fprintf(outfile," - DelayMsg - delaybound equal to 0. Don't increase s.\n");
       fflush(outfile);*/
         return(FALSE);
      }
   } /* else */

} /* DelayMsg */

void CleanPulse(int si)
{
   int z;
   s[pos(si)].delaybound=deltamax+1;
   for (z=0;z<n;z++)
   {
       s[pos(si)].expected[z]=0;
       s[pos(si)].safe[z]=FALSE;
   }
   s[pos(si)].safe[i]=FALSE;
   s[pos(si)].firstmsg=NULL;
   s[pos(si)].lastmsg=NULL;
   /*fprintf(outfile,"%d limpando %d - %d\n",currents,si,pos(si));*/
}
