/*******************************************************
 *	Programa PPC - mo'dulo principal
 * 
 *	(C) 1998-2001, Carlos A.S. Oliveira
 *******************************************************/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <stdlib.h>
#include "common.h"
#include "ppc.h"

/*  -----------------------------
 *  Data structures and defines
 *  ----------------------------- */

#define PROGNAME "PPC"

#define EnableTBButton(bt, opt) \
	SendMessage(gToolbar, TB_ENABLEBUTTON, (WPARAM) bt, \
			(LPARAM) MAKELONG(opt, 0))

#define CheckTBButton(bt, opt) \
	SendMessage(gToolbar,  TB_CHECKBUTTON, (WPARAM)bt, \
			(LPARAM) MAKELONG(opt, 0))

#define SetStatus(text) \
	SendMessage(gStatus, SB_SETTEXT, (WPARAM) (255 | SBT_NOBORDERS),\
			(LPARAM)text)

#define HANDLE_MESSAGE(mess, func) \
  case mess: func; break

#define INFOW_SIZE	100

/*  -----------------------------
 *  global variables
 *  ----------------------------- */

/* instancia do programa */
HINSTANCE ghInst;

/* modo atual do programa (rodando, selecionando vertices, etc.)*/
int gMode = 0;

/* nome do programa e da classe de janela */
static char *strProgName = "Point-to-point Connection";
static char *strWClass = "PPC";

/* numero inicial de vertices (recebido pela linha de comando) */
static int gComLinenVertices;

/* handle da toolbar, barra de status e janela do grafo */
HWND gToolbar = 0, gStatus = 0, gPPC = 0, gCombo = 0, gPPCInfo = 0;

/* procedimentos default da combobox */
WNDPROC gDefCombo, gDefComboEdit, gDefToolbar;

/* handle da janela principal */
static HWND ghWnd = 0;

/* fonte usada pelo programa */
HFONT gPPCFont;

/* pointer to file for saving graphs */
FILE *fout_file = 0;

/*  -----------------------------
 *  function prototypes
 *  ----------------------------- */

BOOL APIENTRY 
SobreFunc(HWND hDlg, UINT message, UINT wParam, LONG lParam);

BOOL APIENTRY 
New_graph_func(HWND hDlg, UINT message, UINT wParam, LONG lParam);

LRESULT CALLBACK 
WinProc(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp);

LRESULT CALLBACK 
ComboWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);

LRESULT CALLBACK 
ToolbarWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);

LRESULT CALLBACK 
ComboEditWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);

int STDCALL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
        int nShowCmd);

static void 
ShowToolTip(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp);

static void
ResizeWindow(HWND hWnd);

static void
InitMainWindow(HWND hWnd);

static int
CreateAppToolBar(HWND hWnd);

static void
handle_file_opensave(HWND hWnd, int what);

static void
set_combo_value(int n);

/*  -----------------------------
 *  Implementation of Functions 
 *  ----------------------------- */


enum open_save_what {
  OS_OPEN,
  OS_SAVE,
  OS_SAVEAS
};

/* 
 *  FUNCTION: handle_file_opensave
 *  PURPOSE: handle menu options open, save and save as
 *  PARAMETERS:
 *	hWnd - janela pai 
 *	what - say what is to do: save, open, or save as
 */
static void
handle_file_opensave(HWND hWnd, int what)
{
  static char file_name[MAX_PATH] = "";
  static OPENFILENAME f;
  if (what == OS_SAVE && strlen(file_name) == 0)
    what = OS_SAVEAS;
  memset(&f, 0, sizeof(f));
  f.lStructSize = sizeof(f);
  f.hwndOwner = hWnd;
  f.hInstance = ghInst;
  f.lpstrFilter = "Graph files\0*.grf\0"
    "Instance files\0*.ins\0";
  f.lpstrCustomFilter = 0;
  f.Flags = OFN_EXPLORER | OFN_OVERWRITEPROMPT;
  f.lpstrDefExt = "grf";
  f.lpstrFile = file_name;
  f.nMaxFile = sizeof(file_name);
  if ((what == OS_SAVEAS && GetSaveFileName(&f) == TRUE)||
      (what == OS_OPEN && GetOpenFileName(&f) == TRUE) ||
      (what == OS_SAVE)) {
    char *mes = "Can not open file ";
    char *error_str;
    if (what == OS_SAVEAS)
      what = OS_SAVE;
    fout_file = fopen(f.lpstrFile, (what == OS_SAVE ? "w" : "r"));
    if (!fout_file) {
      error_str = NewVec(char, sizeof(mes)+sizeof(file_name));
      strcpy(error_str, mes);
      strcat(error_str, file_name);
      MessageBox(hWnd, error_str, "PPC Error", MB_OK);
      Delete(error_str);
    } else {
      if (what == OS_SAVE)
	SendMessage(gPPC, WM_SAVEYOURSELF, 0, (LPARAM)fout_file);
      else
	SendMessage(gPPC, WM_READGRAPH, 0, (LPARAM)fout_file);
    }
  }
}

/* 
 *  FUNCTION: SobreFunc
 *  PURPOSE: procedimento para o quadro de dialogo Sobre
 *  PARAMETERS:
 *	hWnd - janela pai 
 *	mess, wp e lp: parametros enviados a janela
 */
BOOL APIENTRY 
SobreFunc(HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
  switch (message) {
    HANDLE_MESSAGE(WM_INITDIALOG, return (TRUE));
    HANDLE_MESSAGE(WM_CLOSE,
    {
      EndDialog(hDlg, TRUE);
      return (TRUE);
    });
    HANDLE_MESSAGE(WM_COMMAND,
		   if (LOWORD(wParam) == IDOK) {
		     EndDialog(hDlg, TRUE);
		     return (TRUE);
		   });
  }
  return (FALSE);   
}

/* 
 *  FUNCTION: NewGraph function
 *  PURPOSE: window procedure to the new graph dialog
 *  PARAMETERS:
 *	hWnd - parent window
 *	mess, wp e lp: window parameters
 */
BOOL APIENTRY 
New_graph_func(HWND hDlg, UINT message, UINT wParam, LONG lParam)
{
  UDACCEL ua = { 1, 5 };
  HWND sp;
  switch (message) {
    HANDLE_MESSAGE(WM_INITDIALOG,
    {
      sp = GetDlgItem(hDlg, IDC_SPIN);
      /* adjust inicial value in spin control */
      SendMessage(sp, UDM_SETPOS, 0,
		  (LPARAM) MAKELONG((short)gComLinenVertices, 0));
      /* adjust the range */
      SendMessage(sp, UDM_SETRANGE, 0,
		  (LPARAM) MAKELONG((short) UD_MAXVAL-5, (short) 5));
      /* set increment (of 5) */
      SendMessage(sp, UDM_SETACCEL, (WPARAM)1, (LPARAM)(LPUDACCEL)&ua);
      return (TRUE);
    });
    HANDLE_MESSAGE(WM_CLOSE,
    {
      EndDialog(hDlg, TRUE);
      return (TRUE);
    });
    HANDLE_MESSAGE(WM_COMMAND,
    {
      LRESULT res;
      switch (LOWORD(wParam)) {
      case IDOK:
	sp = GetDlgItem(hDlg, IDC_SPIN);
	res = SendMessage(sp, UDM_GETPOS, 0, 0);
	if (HIWORD(res) == 0) {
	  gComLinenVertices = LOWORD(res);
	  set_combo_value(gComLinenVertices);
	  SendMessage(gPPC, WM_CHANGEGRAPH,
		      (WPARAM)gComLinenVertices, 0);
	} else {
	  MessageBox(hDlg, "Wrong number of vertices",
		     "PPC", MB_ICONEXCLAMATION | MB_OK);
	  SetFocus(GetDlgItem(hDlg, IDC_NVERT));
	  /* select all text in the edit box */
	  SendMessage(GetDlgItem(hDlg, IDC_NVERT),
		      EM_SETSEL, 0, (WPARAM)(INT)-1);
	  return 0;
	}
	/* pass through:  */
	/* break */
      case IDCANCEL:
	EndDialog(hDlg, TRUE);
	return (TRUE);
      }
    });
  }
  return (FALSE);   
}

/* 
 *  FUNCTION: handle_wm_esc
 *  PURPOSE: this handle a esc of the user inside the #
 *	verts combo box. It returns the selection to the correct
 *	item in the combo box.
 *  PARAMETERS:
 *	hWnd - parent window
 */
void handle_wm_esc(HWND hWnd)
{
  char achTemp[20];
  DWORD dwIndex;	

  sprintf(achTemp, "%d", gComLinenVertices);
  /* procura a string na lista existente */
  dwIndex = SendMessage(gCombo, CB_FINDSTRINGEXACT, 
			(WPARAM) (-1), (LPARAM) achTemp); 	
  SendMessage(gCombo, CB_SETCURSEL, dwIndex, 0); 
  /* Set the focus to the main window. */
  SetFocus(hWnd); 
}

enum toolbar_status {
  TOST_SELECT
};

void set_toolbar_status(int stat)
{
  switch (stat) {
  case TOST_SELECT:
  CheckTBButton(IDM_GRAPH_SOURCE, FALSE); 
  CheckTBButton(IDM_GRAPH_DEST, FALSE); 
  }
}

void set_combo_value(int n)
{
  DWORD dwIndex;
  char str[20];
  /* try to find the value */
  sprintf(str, "%d", n);
  dwIndex = SendMessage(gCombo, CB_FINDSTRINGEXACT, 
			(WPARAM) (-1), (LPARAM) str);
  if (dwIndex == CB_ERR) /* string not found */
    /* insert it */
    dwIndex = SendMessage(gCombo, CB_ADDSTRING, 
			  0, (LPARAM) str);
  /* select it */
  if (dwIndex != CB_ERR) 
    SendMessage(gCombo, CB_SETCURSEL, dwIndex, 0);
}

int change_graph_size(int nVert)
{
  set_toolbar_status(TOST_SELECT);
  SetStatus("");
  gComLinenVertices = nVert;
  SendMessage(gPPC, WM_CHANGEGRAPH, (WPARAM)nVert, 0);
  return 1;
}

void handle_combo_change(HWND hWnd)
{
  char achTemp[20];
  DWORD dwIndex;
  int res, nVert, not_in_list = 0;
  
  SetFocus(hWnd);
  gMode = STAT_SELECT;
  /* get the current selection */
  dwIndex = SendMessage(gCombo, CB_GETCURSEL, 0, 0);
  if (dwIndex == CB_ERR) {
    /* If no current selection */
    if (SendMessage(gCombo, WM_GETTEXT, sizeof(achTemp), 
		    (LPARAM) achTemp) == 0) {
      /* edit box is empty; set the current value */
      set_combo_value(gComLinenVertices);
      return;
    }
    not_in_list = 1;
  } else {
    SendMessage(gCombo, CB_GETLBTEXT, dwIndex, (LPARAM)(LPCSTR)achTemp);
  }
    
  nVert = atoi(achTemp);
  if (nVert < 2) {
    MessageBox(hWnd, "Invalid number of nodes",
	       PROGNAME, MB_OK | MB_ICONEXCLAMATION);
    set_combo_value(gComLinenVertices);
    return;
  }
  if (nVert == gComLinenVertices) {
    /* nothing to do */
    return;
  }
  res = MessageBox(hWnd, "Do you really want to create\n"
		   "a new graph with different number\n"
		   "of vertices?", strProgName, 
		   MB_YESNO | MB_ICONQUESTION);
  if (res == IDYES){
    /* add the value to the drop list */
    if (not_in_list) {
      set_combo_value(nVert);
    }
    change_graph_size(nVert);
  } else {
    /* return to the old size */
    set_combo_value(gComLinenVertices);
  }
}

void handle_wm_enter(HWND hWnd)
{
  handle_combo_change(hWnd);
}

void handle_wm_command(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp)
{
  switch (wp){
    HANDLE_MESSAGE(IDM_FILE_NEW, DialogBox(ghInst,
					   MAKEINTRESOURCE(5), hWnd,
					   New_graph_func));
    HANDLE_MESSAGE(IDM_FILE_OPEN,handle_file_opensave(hWnd, OS_OPEN));
    HANDLE_MESSAGE(IDM_FILE_SAVE,handle_file_opensave(hWnd, OS_SAVE));
    HANDLE_MESSAGE(IDM_FILE_SAVEAS,handle_file_opensave(hWnd, OS_SAVEAS));
    HANDLE_MESSAGE(IDM_FILE_EXIT, SendMessage(hWnd, WM_CLOSE, 0, 0));
    HANDLE_MESSAGE(IDM_HELP_SOBRE,DialogBox(ghInst,
					    MAKEINTRESOURCE(1), hWnd,
					    SobreFunc));
    HANDLE_MESSAGE(CBN_CLOSEUP, handle_combo_change(hWnd));
  case IDM_GRAPH_REDEFINE:
    gMode = STAT_SELECT;
    CheckTBButton(IDM_GRAPH_SOURCE, FALSE); 
    CheckTBButton(IDM_GRAPH_DEST, FALSE); 
    SetStatus("");
    if (MessageBox(hWnd, "Do you really want to\n"
		   "create a new graph?", strProgName, 
		   MB_YESNO | MB_ICONQUESTION)== IDYES){
      SendMessage(gPPC, WM_CHANGEGRAPH, (WPARAM)gComLinenVertices, 0);
    }
    break;

  case IDM_GRAPH_SOURCE:
    gMode = STAT_DRAW_SOURCE;
    SetStatus("Clique no vrtice fonte");
    break;

  case IDM_GRAPH_DEST:
    gMode = STAT_DRAW_DEST;
    SetStatus("Clique no vrtice destino");
    break;

  case IDM_GRAPH_EXECUTE:
    gMode = STAT_SELECT;
    CheckTBButton(IDM_GRAPH_SOURCE, FALSE); 
    CheckTBButton(IDM_GRAPH_DEST, FALSE); 
    SetStatus("");

    /* Desabilita alguns botoes da barra de status */
    if (SendMessage(gPPC, WM_EXECUTE, (WPARAM) 0, (LPARAM) 0)){
      EnableTBButton(IDM_GRAPH_REDEFINE, FALSE); 
      EnableTBButton(IDM_GRAPH_SOURCE, FALSE);
      EnableTBButton(IDM_GRAPH_DEST, FALSE);
      EnableTBButton(IDM_GRAPH_EXECUTE, FALSE);
      EnableWindow(gCombo, FALSE); 	
    }
    break;

  case IDM_GRAPH_STOP:
    gMode = STAT_SELECT;
    if (SendMessage(gPPC, WM_STOPATEAMS, (WPARAM) 0, (LPARAM) 0)){
      EnableTBButton(IDM_GRAPH_REDEFINE, TRUE); 
      EnableTBButton(IDM_GRAPH_SOURCE, TRUE);
      EnableTBButton(IDM_GRAPH_DEST, TRUE);
      EnableTBButton(IDM_GRAPH_EXECUTE, TRUE);
      EnableTBButton(IDM_GRAPH_STOP, FALSE);
      EnableWindow(gCombo, TRUE); 	
      InvalidateRect(gPPCInfo, NULL, TRUE);
    }
    break;

  case IDM_COLOR_SOURCE:
  case IDM_COLOR_DEST:
  case IDM_COLOR_NODE:
  case IDM_COLOR_LINE:
  case IDM_COLOR_SOL:
  case IDM_COLOR_BACK:
  case IDM_COLOR_DEFAULT:
    SendMessage(gPPC, WM_COMMAND, wp, 0);
    break;
    
  }
}

/* 
 *  FUNCTION: WinProc
 *  PURPOSE: Procedimento da janela principal
 *  PARAMETERS:
 *	hWnd - janela pai 
 *	mess, wp e lp: parametros enviados a janela
 */
LRESULT CALLBACK 
WinProc(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp)
{
  switch (mess){
    HANDLE_MESSAGE(WM_CREATE, InitMainWindow(hWnd));
    HANDLE_MESSAGE(WM_DESTROY, DeleteObject(gPPCFont));
    HANDLE_MESSAGE(WM_SIZE, ResizeWindow(hWnd));
    /* mensagem enviada pela combobox */
    HANDLE_MESSAGE(WM_ESC, handle_wm_esc(hWnd));
    HANDLE_MESSAGE(WM_ENTER, handle_wm_enter(hWnd));
    HANDLE_MESSAGE(WM_COMMAND, handle_wm_command(hWnd, mess, wp, lp));

  case WM_NOTIFY:
    switch (((NMHDR *)lp)->code) { 
      /* O tooltip esta' pedindo uma string para apresentar */
      HANDLE_MESSAGE(TTN_NEEDTEXT, ShowToolTip(hWnd, mess, wp, lp));
    } 
    break;
  case WM_END_FILE_OPERATION:
    /* ppcw saved the graph file. */
    if (fout_file) {
      fclose(fout_file);
      fout_file = 0;
    }
    break;
  case WM_CLOSE:
    if (MessageBox(hWnd, "Do you really want to exit?",
		   strProgName,
		   MB_YESNO | MB_ICONQUESTION) == IDYES){
      PostQuitMessage(0);
    } else return 0;
    break;
  default:
    return DefWindowProc(hWnd, mess, wp, lp);
  }
  return 0;        
}


/* 
 *  FUNCTION: WinMain
 *  PURPOSE: Registra a classe de janela, cria a janela principal
 *	e inicia  o laco de mensagens.	
 */
int STDCALL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
        int nShowCmd)
{
  HWND wnd;
  MSG msg;
  WNDCLASS wc;
	
  gComLinenVertices = atoi(lpCmdLine);
  /* recebe como parametro o numero de vertices do grafo */
  if (gComLinenVertices < 2)
    gComLinenVertices = 100;
  /* guarda o handle da instancia */
  ghInst = hInstance;

  /* preenche a estrutura da classe de janela */
  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = WinProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = ghInst;
  wc.hIcon = LoadIcon(ghInst, MAKEINTRESOURCE(2));
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = GetStockObject (WHITE_BRUSH);
  wc.lpszMenuName = MAKEINTRESOURCE(4);
  wc.lpszClassName = strWClass;

  /* Registra a janela */
  RegisterClass(&wc);
  InitCommonControls();
  /* cria a janela */
  wnd = CreateWindow(strWClass, strProgName,
		     WS_OVERLAPPEDWINDOW,
		     CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
		     CW_USEDEFAULT, NULL, NULL, ghInst, NULL);
  ShowWindow (wnd, nShowCmd);
  UpdateWindow (wnd);
	
  /* laco principal de mensagens */
  while (GetMessage(&msg, NULL, 0,0)){
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

/* 
 *  FUNCTION: ShowToolTip
 *  PURPOSE: Fornece os textos que serao mostrados pelo tooltip, conforme requisitado
 *	pelo sistema. a string e' devolvida como um ponteiro em uma estrutura 
 *	LPTOOLTIPTEXT que e' armazenada em lp 
 *  PARAMETERS:
 *	hWnd - janela pai 
 *	mess, wp e lp: parametros enviados a janela
 */
void 
ShowToolTip(HWND hWnd, UINT mess, WPARAM wp, LPARAM lp)
{
  LPTOOLTIPTEXT lpToolTipText;
  char szBuf[255];
  lpToolTipText = (LPTOOLTIPTEXT)lp;
  if (lpToolTipText->hdr.code ==  (unsigned)TTN_NEEDTEXT){
    /* carrega o texto de um recurso. */
    LoadString (ghInst, 
		lpToolTipText->hdr.idFrom,    /* string ID == cmd ID */
		szBuf, sizeof(szBuf));
    /* aponta para a string que sera mostrada. */
    lpToolTipText->lpszText = szBuf;
  }
}

/* 
 *  FUNCTION: ResizeWindow
 *  PURPOSE: altera o tamanho das janelas-filho quando a janela principal
 *	muda de tamanho 
 *  PARAMETERS:
 *	hWnd - janela pai 
 */
void
ResizeWindow(HWND hWnd)
{
  RECT lpr;
  int h1, h2;
  HDWP hdwp; 
	
  /* reposiciona a barra de ferramentas */
  SendMessage (gToolbar, TB_AUTOSIZE, (WPARAM)0L, (LPARAM)0L);

  /* pega as alturas das barras de status e ferramentas */		
  GetWindowRect(gToolbar, &lpr);
  h1 = lpr.bottom - lpr.top;	
  GetWindowRect(gStatus, &lpr);
  h2 = lpr.bottom - lpr.top;

  /* pega a area cliente da janela  */
  GetClientRect(hWnd, &lpr);

  hdwp = BeginDeferWindowPos(3); 
  /* reposiciona a barra de status */
  DeferWindowPos(hdwp, gStatus,
		 NULL, lpr.left, lpr.bottom - h2,
		 lpr.right - lpr.left, h2, SWP_NOZORDER); 
  /* reposiciona a janela de informacoes */
  DeferWindowPos(hdwp, gPPCInfo,
		 NULL, lpr.left, lpr.top + h1,
		 lpr.left + INFOW_SIZE, lpr.bottom - h1 - h2, SWP_NOZORDER); 
  /* reposiciona a janela do grafo */
  DeferWindowPos(hdwp, gPPC,
		 NULL, lpr.left + INFOW_SIZE + 1, lpr.top + h1,
		 lpr.right - lpr.left - INFOW_SIZE - 1, lpr.bottom - h1 - h2, SWP_NOZORDER); 
  EndDeferWindowPos(hdwp); 

}

/* 
 *  FUNCTION: InitMainWindow
 *  PURPOSE: Inicializa a janela principal.
 *	Cria as janelas filhas (barra de estatus de ferramentas e janela
 *	do grafo 
 *  PARAMETERS:
 *	hWnd - janela pai 
 */
void
InitMainWindow(HWND hWnd)
{
  RECT r;
  int h1, h2, w;
  LOGFONT plf;

  memset(&plf, 0, sizeof(LOGFONT));
  plf.lfHeight = 15;
  strcpy(plf.lfFaceName, "Arial"); 
  plf.lfWeight = FW_NORMAL; 
  plf.lfEscapement = 0;
  gPPCFont = CreateFontIndirect(&plf);

  GetClientRect(hWnd, &r);
  h2 = r.bottom;
  w = r.right;

  ghWnd = hWnd;
  gMode = STAT_SELECT;
  if (!CreateAppToolBar(hWnd))
    MessageBox (hWnd, "Erro na criacao da barra de botes!", 
		strProgName, MB_OK );
  GetWindowRect(gToolbar, &r);
  h1 = r.bottom;

  /* 
   * cria a barra de estatus 
   */
  gStatus = CreateStatusWindow(WS_CHILD | WS_VISIBLE, "",
			       hWnd, 0);
  if (!gStatus)
    MessageBox (hWnd, "Erro na criacao da barra de status!", 
		strProgName, MB_OK );
  SendMessage(gStatus, SB_SIMPLE, (WPARAM)TRUE, 0);
  GetWindowRect(gStatus, &r);
  h2 -= r.bottom - r.top -1;

  /* 
   * cria a janela de informacoes.  
   */
  RegisterPPCInfo();
  gPPCInfo = CreatePPCInfo(hWnd, 0, h1, INFOW_SIZE, h2);
	

  /* 
   * cria a janela de grafo.  
   */
  RegisterPPCW();
  gPPC = CreatePPCW(hWnd, INFOW_SIZE + 1, h1, w - INFOW_SIZE - 1, 
		    h2, gComLinenVertices); 
  if (!gPPC)
    MessageBox (hWnd, "Erro na janela do grafo!", 
		strProgName, MB_OK );
}


/* 
 *  FUNCTION: CreateAppToolBar
 *  PURPOSE: Create the toolbar of the application
 * 	and initialize its components
 *  PARAMETERS:
 *	hWnd - parent window
 */
int
CreateAppToolBar(HWND hWnd)
{
  /* structure that describe the buttons in the tool bar */
  TBBUTTON tbButtons[] = {
    { 0, IDM_GRAPH_REDEFINE, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0L, 0},
    { 0, 0, 0, TBSTYLE_SEP, 0L, 0},
    { 1, IDM_GRAPH_SOURCE, TBSTATE_ENABLED, 
      TBSTYLE_CHECKGROUP, 0L, 0},
    { 2, IDM_GRAPH_DEST, TBSTATE_ENABLED, 
      TBSTYLE_CHECKGROUP, 0L, 0},
    { 0, 0, 0, TBSTYLE_SEP, 0L, 0},
    { 3, IDM_GRAPH_EXECUTE, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0L, 0},
    { 4, IDM_GRAPH_STOP, 0, TBSTYLE_BUTTON, 0L, 0},
    { 5, IDM_HELP_SOBRE, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0L, 0},
    { 6, IDM_FILE_EXIT, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0L, 0}};
  /* Declare a bitmap structure */
  TBADDBITMAP tbBitmaps;
  HWND hWndTT, editCombo, label;
  /* strings to be added to the combobox */
  char *cbstr[10] = {
    "10", "20", "30", "40", "50", "60", "70", "80", "90", "100"
  };
  char szBuf[255];
  TOOLINFO lpToolInfo;
  POINT pt;
  int i;

  /* init members of the bitmap structure */
  tbBitmaps.hInst = ghInst; 
  /* bitmap resource ID */
  tbBitmaps.nID = 3;  
  /* create the bar */
  gToolbar = CreateToolbarEx(hWnd, 
			     WS_CHILD | WS_BORDER | WS_VISIBLE |	TBSTYLE_TOOLTIPS, 
			     0,                  /* ID da barra de ferramentas      */
			     7,                  /* numero de imagens de botoes     */
			     ghInst,             /* instancia que possui os bitmaps */
			     3,                  /* ID do recurso de bitmap         */
			     tbButtons,          /* estrutura TBBUTTON              */
			     9,                  /* numero de botoes                */ 
			     15,15,15,15,        /* largura e altura dos botoes e   */
			                         /* suas imagens                    */
			     sizeof(TBBUTTON));  /* tamanho da estrutura TBUTTON    */
  if (!gToolbar){
    return 0;
  }

  /* create a label */
  label = CreateWindow("STATIC", "# vertices:",
		       WS_CHILD | WS_VISIBLE | SS_RIGHT,
		       180, 3, 60, 14, gToolbar, 0, ghInst, NULL);

  SendMessage(label, WM_SETFONT, (WPARAM)gPPCFont, MAKELPARAM(TRUE, 0));

  /* Create a combobox and add it to the bar */
  gCombo = CreateWindowEx ( 0L,"COMBOBOX",                            /* name of the class */
			    "10",                                     /* default text      */
			    WS_CHILD | WS_BORDER | WS_VISIBLE |
			    CBS_HASSTRINGS | CBS_DROPDOWN,            /* window style      */
			    245, 0, 50, 250,                          /* size and position */
			    gToolbar,                                 /* parent window     */
			    0, ghInst, NULL );


  /* Pega  o handle do campo de edicao da combo box. */  
  pt.x = 5; 
  pt.y = 5; 
  editCombo = ChildWindowFromPoint(gCombo, pt); 

  /* redefine o procedimento de janela para a combo box. */
  gDefCombo = (WNDPROC)GetWindowLong(gCombo, GWL_WNDPROC);
  SetWindowLong(gCombo, GWL_WNDPROC, (LONG)ComboWndProc);


  /* redefine o procedimento de janela para o campo de edicao. */
  gDefComboEdit = (WNDPROC)GetWindowLong(editCombo, GWL_WNDPROC);
  SetWindowLong(editCombo, GWL_WNDPROC, (LONG)ComboEditWndProc);

  /* redefine o procedimento de janela para o tool box. */
  gDefToolbar = (WNDPROC)GetWindowLong(gToolbar, GWL_WNDPROC);
  SetWindowLong(gToolbar, GWL_WNDPROC, (LONG)ToolbarWndProc);

  if (gCombo){
    /* Adiciona strings `a combobox */
    for (i=0; i < 10; i++)
      SendMessage(gCombo, CB_INSERTSTRING,(WPARAM)(-1), 
		  (LPARAM)cbstr[i]);
  }
	
  /* seleciona um item */
  SendMessage(gCombo, CB_SETCURSEL, (WPARAM)9, 0L);
	

  /* Guarda o handle do ToolTip */
  hWndTT = (HWND)SendMessage(gToolbar, TB_GETTOOLTIPS, (WPARAM)0, (LPARAM)0);

  if (!hWndTT){
    return 0; 
  }

  /* preenche a estrutura TOOLINFO */
  lpToolInfo.cbSize = sizeof(lpToolInfo);
  lpToolInfo.uFlags = TTF_IDISHWND;
  /* carrega o texto de um recurso. */
  LoadString (ghInst, IDM_GRAPH_NUMVERT, szBuf, sizeof(szBuf));
  lpToolInfo.lpszText = szBuf;
  /* janela que recebe menssagens do ToolTip */
  lpToolInfo.hwnd = hWnd;
  /* ferramenta que sera' inserida (o combobox) */
  lpToolInfo.uId = (UINT)gCombo;
  lpToolInfo.hinst = 0;         /* nao e' usado */

  /* adiciona a ferramenta `a combo box. */
  SendMessage(hWndTT, TTM_ADDTOOL, (WPARAM)0, (LPARAM)(LPTOOLINFO)&lpToolInfo);

  return 1;
}


/* 
 *  FUNCTION: ComboWndProc
 *  PURPOSE: Procedimento de janela que e' usado para 
 * 	customizar a classe de janela combobox. Pass as mensagens relacionadas
 *	ao mouse para a janela de tooltip
 *  PARAMETERS:
 *	hWnd - janela pai 
 *    uMessage, wParam e lParam: parametros da mensagem
 */
LRESULT CALLBACK 
ComboWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
  switch (uMessage){
  case WM_MOUSEMOVE:
  case WM_LBUTTONDOWN:
  case WM_LBUTTONUP: 
    {
      MSG msg;
      HWND hWndTT;
      msg.lParam = lParam;
      msg.wParam = wParam;
      msg.message = uMessage;
      msg.hwnd = hWnd;
      /* pega o handle da janela de tooltips */
      hWndTT = (HWND)SendMessage(gToolbar, TB_GETTOOLTIPS, (WPARAM)0,(LPARAM)0);
	    SendMessage(hWndTT, TTM_RELAYEVENT, (WPARAM)0, (LPARAM)(LPMSG)&msg);
    }
    break;
  }
  /* chama o procedimento de janela original */
  return (CallWindowProc(gDefCombo, hWnd, uMessage, wParam, lParam));
}

LRESULT CALLBACK 
ToolbarWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
  switch (uMessage){
  case WM_COMMAND:
    switch (HIWORD(wParam)) {
      HANDLE_MESSAGE(CBN_SELCHANGE, handle_combo_change(ghWnd));
    }
    break;
  }
  /* chama o procedimento de janela original */
  return (CallWindowProc(gDefToolbar, hWnd, uMessage, wParam, lParam));
}


/* 
 *  FUNCTION: ComboEditWndProc
 *  PURPOSE: Procedimento de janela que e' usado para 
 * 	customizar o campo de edicao da combobox. Pass as mensagens relacionadas
 *	ao teclado para a janela principal
 *  PARAMETERS:
 *	hWnd - janela pai 
 *    uMessage, wParam e lParam: parametros da mensagem
 */
LRESULT CALLBACK 
ComboEditWndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
  switch (uMessage){
  case WM_KEYDOWN: 
    switch (wParam) { 
    case VK_ESCAPE: 
      SendMessage(ghWnd, WM_ESC, 0, 0); 
      return 0; 
    case VK_RETURN: 
      SendMessage(ghWnd, WM_ENTER, 0, 0); 
      return 0; 
    } 
    break; 
 
  case WM_KEYUP: 
  case WM_CHAR: 
    switch (wParam) { 
    case VK_ESCAPE: 
    case VK_RETURN: 
      return 0; 
    } 
    break; 
  }
  /* chama o procedimento de janela original */
  return (CallWindowProc(gDefComboEdit, hWnd, uMessage, wParam, lParam));
}

/* 
 *  FUNCTION: RedrawPPCWindow
 *  PURPOSE: Usado para atualizar o conteudo da janela do grafo.
 *  PARAMETERS:
 *	void
 */
void 
RedrawPPCWindow()
{
  InvalidateRect(gPPC, NULL, TRUE); 
}

void
RedrawInfo()
{
  SendMessage(gPPCInfo, WM_UPDATEINFO, 0, 0); 
}

/* 
 *  FUNCTION: DebugInt
 *  PURPOSE: mostra um valor inteiro apenas para depuracao.
 *  PARAMETERS:
 *	void
 */
void 
DebugInt(int i)
{ 
  char s[50];
  sprintf(s, "valor = %d", i);
  MessageBox(0, s, strProgName, IDOK);
}

void
EnablePPCButton(int bt, int opt)
{
  SendMessage(gToolbar, TB_ENABLEBUTTON, (WPARAM) bt, 
			(LPARAM) MAKELONG(opt, 0));
}

